@@ -2,8 +2,8 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2016 Andrew Ziem
-# http://www.bleachbit.org
+# Copyright (C) 2008-2018 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,30 +18,28 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-
 """
 Launcher
 """
 
+from __future__ import absolute_import, print_function
 
 import os
 import sys
 
-if 'nt' == os.name:
-    from bleachbit.Windows import setup_environment
-    setup_environment()
+if 'posix' == os.name:
+    if os.path.isdir('/usr/share/bleachbit'):
+        # This path contains bleachbit/{C,G}LI.py .  This section is
+        # unnecessary if installing BleachBit in site-packages.
+        sys.path.append('/usr/share/')
 
-if 'posix' == os.name and os.path.isdir('/usr/share/bleachbit'):
-    # This path contains bleachbit/{C,G}LI.py .  This section is
-    # unnecessary if installing BleachBit in site-packages.
-    sys.path.append('/usr/share/')
 
 if 1 == len(sys.argv):
     import gtk
     try:
         gtk.gdk.Screen().get_display()
     except RuntimeError:
-        print "Could not open X display"
+        print("Could not open X display")
         sys.exit(1)
     import bleachbit.GUI
     gui = bleachbit.GUI.GUI()
