@@ -2,8 +2,8 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2015 Andrew Ziem
-# http://bleachbit.sourceforge.net
+# Copyright (C) 2008-2018 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,20 +23,23 @@
 Actions that perform cleaning
 """
 
+from __future__ import absolute_import, print_function
+
+from bleachbit import Command, FileUtilities, General, Special
+from bleachbit import _, expanduser, expandvars
 
 import glob
+import logging
 import os
 import re
 import types
-import Command
-import FileUtilities
-import General
-import Special
 
-from Common import _
 
 if 'posix' == os.name:
-    import Unix
+    re_flags = 0
+    from bleachbit import Unix
+else:
+    re_flags = re.IGNORECASE
 
 
 #
@@ -86,9 +89,13 @@ class FileActionProvider(ActionProvider):
         assert(isinstance(self.regex, (str, unicode, types.NoneType)))
         self.nregex = action_element.getAttribute('nregex')
         assert(isinstance(self.nregex, (str, unicode, types.NoneType)))
+        self.wholeregex = action_element.getAttribute('wholeregex')
+        assert(isinstance(self.wholeregex, (str, unicode, types.NoneType)))
+        self.nwholeregex = action_element.getAttribute('nwholeregex')
+        assert(isinstance(self.nwholeregex, (str, unicode, types.NoneType)))
         self.search = action_element.getAttribute('search')
-        self.path = os.path.expanduser(os.path.expandvars(
-            action_element.getAttribute('path')))
+        self.object_type = action_element.getAttribute('type')
+        self.path = expanduser(expandvars(action_element.getAttribute('path')))
         if 'nt' == os.name and self.path:
             # convert forward slash to backslash for compatibility with getsize()
             # and for display.  Do not convert an empty path, or it will become
@@ -102,13 +109,52 @@ class FileActionProvider(ActionProvider):
                 action_element.getAttribute('cache'))
             self.ds['command'] = action_element.getAttribute('command')
             self.ds['path'] = self.path
+        if not any([self.object_type, self.regex, self.nregex,
+                    self.wholeregex, self.nwholeregex]):
+            # If the filter is not needed, bypass it for speed.
+            self.get_paths = self._get_paths
 
     def get_deep_scan(self):
         if 0 == len(self.ds):
             raise StopIteration
         yield self.ds
 
+    def path_filter(self, path):
+        """Process the filters: regex, nregex, type
+
+        If a filter is defined and it fails to match, this function
+        returns False. Otherwise, this function returns True."""
+
+        if self.regex:
+            if not self.regex_c.search(os.path.basename(path)):
+                return False
+
+        if self.nregex:
+            if self.nregex_c.search(os.path.basename(path)):
+                return False
+
+        if self.wholeregex:
+            if not self.wholeregex_c.search(path):
+                return False
+
+        if self.nwholeregex:
+            if self.nwholeregex_c.search(path):
+                return False
+
+        if self.object_type:
+            if 'f' == self.object_type and not os.path.isfile(path):
+                return False
+            elif 'd' == self.object_type and not os.path.isdir(path):
+                return False
+
+        return True
+
     def get_paths(self):
+        import itertools
+        for f in itertools.ifilter(self.path_filter, self._get_paths()):
+            yield f
+
+    def _get_paths(self):
         """Return a filtered list of files"""
 
         def get_file(path):
@@ -117,7 +163,8 @@ class FileActionProvider(ActionProvider):
 
         def get_walk_all(top):
             for expanded in glob.iglob(top):
-                for path in FileUtilities.children_in_directory(expanded, True):
+                for path in FileUtilities.children_in_directory(
+                        expanded, True):
                     yield path
 
         def get_walk_files(top):
@@ -139,34 +186,19 @@ class FileActionProvider(ActionProvider):
             raise RuntimeError("invalid search='%s'" % self.search)
 
         if self.regex:
-            regex_c = re.compile(self.regex)
+            self.regex_c = re.compile(self.regex, re_flags)
 
         if self.nregex:
-            nregex_c = re.compile(self.nregex)
+            self.nregex_c = re.compile(self.nregex, re_flags)
 
-        # Sometimes this loop repeats many times, so optimize it by
-        # putting the conditional outside the loop.
+        if self.wholeregex:
+            self.wholeregex_c = re.compile(self.wholeregex, re_flags)
 
-        if not self.regex and not self.nregex:
-            for path in func(self.path):
-                yield path
-        elif self.regex and not self.nregex:
-            for path in func(self.path):
-                if not regex_c.search(os.path.basename(path)):
-                    continue
-                yield path
-        elif self.nregex:
-            for path in func(self.path):
-                if nregex_c.search(os.path.basename(path)):
-                    continue
-                yield path
-        else:
-            for path in func(self.path):
-                if not regex_c.search(os.path.basename(path)):
-                    continue
-                if nregex_c.search(os.path.basename(path)):
-                    continue
-                yield path
+        if self.nwholeregex:
+            self.nwholeregex_c = re.compile(self.nwholeregex, re_flags)
+
+        for path in func(self.path):
+            yield path
 
     def get_commands(self):
         raise NotImplementedError('not implemented')
@@ -209,6 +241,22 @@ class AptAutoremove(ActionProvider):
                                    'apt-get autoremove')
 
 
+class AptClean(ActionProvider):
+
+    """Action to run 'apt-get clean'"""
+    action_key = 'apt.clean'
+
+    def __init__(self, action_element):
+        pass
+
+    def get_commands(self):
+        # Checking executable allows auto-hide to work for non-APT systems
+        if FileUtilities.exe_exists('apt-get'):
+            yield Command.Function(None,
+                                   Unix.apt_clean,
+                                   'apt-get clean')
+
+
 class ChromeAutofill(FileActionProvider):
 
     """Action to clean 'autofill' table in Google Chrome/Chromium"""
@@ -301,6 +349,18 @@ class Ini(FileActionProvider):
             yield Command.Ini(path, self.section, self.parameter)
 
 
+class Journald(ActionProvider):
+    """Action to run 'journalctl --vacuum-time=1'"""
+    action_key = 'journald.clean'
+
+    def __init__(self, action_element):
+        pass
+
+    def get_commands(self):
+        if FileUtilities.exe_exists('journalctl'):
+            yield Command.Function(None, Unix.journald_clean, 'journalctl --vacuum-time=1')
+
+
 class Json(FileActionProvider):
 
     """Action to clean JSON configuration files"""
@@ -338,6 +398,42 @@ class OfficeRegistryModifications(FileActionProvider):
                 _('Clean'))
 
 
+class Process(ActionProvider):
+
+    """Action to run a process"""
+    action_key = 'process'
+
+    def __init__(self, action_element):
+        self.cmd = expandvars(action_element.getAttribute('cmd'))
+        # by default, wait
+        self.wait = True
+        wait = action_element.getAttribute('wait')
+        if wait and wait.lower()[0] in ('f', 'n'):
+            # false or no
+            self.wait = False
+
+    def get_commands(self):
+
+        def run_process():
+            try:
+                if self.wait:
+                    args = self.cmd.split(' ')
+                    (rc, stdout, stderr) = General.run_external(args)
+                else:
+                    rc = 0  # unknown because we don't wait
+                    from subprocess import Popen
+                    Popen(self.cmd)
+            except Exception as e:
+                raise RuntimeError(
+                    'Exception in external command\nCommand: %s\nError: %s' % (self.cmd, str(e)))
+            else:
+                if not 0 == rc:
+                    logging.getLogger(__name__).warning('Command: %s\nReturn code: %d\nStdout: %s\nStderr: %s\n',
+                                                        self.cmd, rc, stdout, stderr)
+            return 0
+        yield Command.Function(path=None, func=run_process, label=_("Run external command: %s") % self.cmd)
+
+
 class Shred(FileActionProvider):
 
     """Action to shred files (override preference)"""
@@ -381,7 +477,7 @@ class WinShellChangeNotify(ActionProvider):
     action_key = 'win.shell.change.notify'
 
     def get_commands(self):
-        import Windows
+        from bleachbit import Windows
         yield Command.Function(
             None,
             Windows.shell_change_notify,
