@@ -2,8 +2,8 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2016 Andrew Ziem
-# http://www.bleachbit.org
+# Copyright (C) 2008-2018 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,25 +18,25 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-
 """
 Command line interface
 """
 
+from __future__ import absolute_import, print_function
 
+from bleachbit.Cleaner import backends, create_simple_cleaner, register_cleaners
+from bleachbit import _, APP_VERSION, encoding
+from bleachbit import Diagnostic, Options, Worker
+
+import logging
 import optparse
 import os
 import sys
 
-from Cleaner import backends, create_simple_cleaner, register_cleaners
-from Common import _, APP_VERSION, encoding
-import Diagnostic
-import Options
-import Worker
+logger = logging.getLogger(__name__)
 
 
 class CliCallback:
-
     """Command line's callback passed to Worker"""
 
     def __init__(self):
@@ -48,7 +48,7 @@ class CliCallback:
         # If the encoding is not explicitly handled on a non-UTF-8
         # system, then special Latin-1 characters such as umlauts may
         # raise an exception as an encoding error.
-        print msg.strip('\n').encode(self.encoding, 'replace')
+        print(msg.strip('\n').encode(self.encoding, 'replace'))
 
     def update_progress_bar(self, status):
         """Not used"""
@@ -79,7 +79,7 @@ def cleaners_list():
 def list_cleaners():
     """Display available cleaners"""
     for cleaner in cleaners_list():
-        print cleaner
+        print (cleaner)
 
 
 def preview_or_clean(operations, really_clean):
@@ -103,23 +103,23 @@ def args_to_operations(args, preset):
                     args.append('.'.join([c_id, o_id]))
     for arg in args:
         if 2 != len(arg.split('.')):
-            print _("not a valid cleaner: %s") % arg
+            logger.warning(_("not a valid cleaner: %s"), arg)
             continue
         (cleaner_id, option_id) = arg.split('.')
         # enable all options (for example, firefox.*)
         if '*' == option_id:
-            if operations.has_key(cleaner_id):
+            if cleaner_id in operations:
                 del operations[cleaner_id]
             operations[cleaner_id] = []
             for (option_id2, o_name) in backends[cleaner_id].get_options():
                 operations[cleaner_id].append(option_id2)
             continue
         # add the specified option
-        if not operations.has_key(cleaner_id):
+        if cleaner_id not in operations:
             operations[cleaner_id] = []
-        if not option_id in operations[cleaner_id]:
+        if option_id not in operations[cleaner_id]:
             operations[cleaner_id].append(option_id)
-    for (k, v) in operations.iteritems():
+    for (k, v) in operations.items():
         operations[k] = sorted(v)
     return operations
 
@@ -129,7 +129,7 @@ def process_cmd_line():
     # TRANSLATORS: This is the command line usage.  Don't translate
     # %prog, but do translate usage, options, cleaner, and option.
     # More information about the command line is here
-    # http://www.bleachbit.org/documentation/command-line
+    # https://www.bleachbit.org/documentation/command-line
     usage = _("usage: %prog [options] cleaner.option1 cleaner.option2")
     parser = optparse.OptionParser(usage)
     parser.add_option("-l", "--list-cleaners", action="store_true",
@@ -146,6 +146,8 @@ def process_cmd_line():
                       help=_("show system information"))
     parser.add_option("--gui", action="store_true",
                       help=_("launch the graphical interface"))
+    parser.add_option('--exit', action='store_true',
+                      help=optparse.SUPPRESS_HELP)
     if 'nt' == os.name:
         uac_help = _("do not prompt for administrator privileges")
     else:
@@ -153,6 +155,8 @@ def process_cmd_line():
     parser.add_option("--no-uac", action="store_true", help=uac_help)
     parser.add_option("-p", "--preview", action="store_true",
                       help=_("preview files to be deleted and other changes"))
+    parser.add_option('--pot', action='store_true',
+                      help=optparse.SUPPRESS_HELP)
     parser.add_option("--preset", action="store_true",
                       help=_("use options set in the graphical interface"))
     if 'nt' == os.name:
@@ -165,22 +169,20 @@ def process_cmd_line():
     (options, args) = parser.parse_args()
     did_something = False
     if options.debug_log:
-        import logging
-        logger = logging.getLogger(__name__)
         logger.addHandler(logging.FileHandler(options.debug_log))
-        logger.info('BleachBit version %s' % APP_VERSION)
+        logger.info('BleachBit version %s', APP_VERSION)
         logger.info(Diagnostic.diagnostic_info())
     if options.version:
-        print """
+        print("""
 BleachBit version %s
-Copyright (C) 2008-2016 Andrew Ziem.  All rights reserved.
+Copyright (C) 2008-2018 Andrew Ziem.  All rights reserved.
 License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.
 This is free software: you are free to change and redistribute it.
-There is NO WARRANTY, to the extent permitted by law.""" % APP_VERSION
+There is NO WARRANTY, to the extent permitted by law.""" % APP_VERSION)
         sys.exit(0)
     if 'nt' == os.name and options.update_winapp2:
-        import Update
-        print "Checking online for updates to winapp2.ini"
+        from bleachbit import Update
+        logger.info("Checking online for updates to winapp2.ini")
         Update.check_updates(False, True,
                              lambda x: sys.stdout.write("%s\n" % x),
                              lambda: None)
@@ -189,27 +191,35 @@ There is NO WARRANTY, to the extent permitted by law.""" % APP_VERSION
     if options.list_cleaners:
         list_cleaners()
         sys.exit(0)
+    if options.pot:
+        from bleachbit.CleanerML import create_pot
+        create_pot()
+        sys.exit(0)
     if options.preview or options.clean:
         operations = args_to_operations(args, options.preset)
         if not operations:
-            print 'ERROR: No work to do.  Specify options.'
+            logger.error('No work to do. Specify options.')
             sys.exit(1)
     if options.preview:
         preview_or_clean(operations, False)
         sys.exit(0)
     if options.overwrite:
         if not options.clean or options.shred:
-            print 'NOTE: --overwrite is intended only for use with --clean'
+            logger.warning('--overwrite is intended only for use with --clean')
         Options.options.set('shred', True, commit=False)
     if options.clean:
         preview_or_clean(operations, True)
         sys.exit(0)
     if options.gui:
         import gtk
-        import GUI
+        from bleachbit import GUI
         shred_paths = args if options.shred else None
-        GUI.GUI(uac=not options.no_uac, shred_paths=shred_paths)
+        GUI.GUI(uac=not options.no_uac,
+                shred_paths=shred_paths, exit=options.exit)
         gtk.main()
+        if options.exit:
+            # For automated testing of Windows build
+            print('Success')
         sys.exit(0)
     if options.shred:
         # delete arbitrary files without GUI
@@ -219,7 +229,7 @@ There is NO WARRANTY, to the extent permitted by law.""" % APP_VERSION
         preview_or_clean(operations, True)
         sys.exit(0)
     if options.sysinfo:
-        print Diagnostic.diagnostic_info()
+        print(Diagnostic.diagnostic_info())
         sys.exit(0)
     if not did_something:
         parser.print_help()
