@@ -1,8 +1,8 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2015 Andrew Ziem
-# http://bleachbit.sourceforge.net
+# Copyright (C) 2008-2018 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,28 +22,24 @@
 Perform (or assist with) cleaning operations.
 """
 
+from __future__ import absolute_import, print_function
+
+from bleachbit import _, expanduser, expandvars
+from bleachbit.FileUtilities import children_in_directory
+from bleachbit.Options import options
+from bleachbit import Command, FileUtilities, Memory, Special
 
 import glob
 import logging
 import os.path
 import re
 import sys
-import traceback
 import warnings
 
-import Command
-import FileUtilities
-import Memory
-import Special
-
 if 'posix' == os.name:
-    import Unix
+    from bleachbit import Unix
 elif 'nt' == os.name:
-    import Windows
-
-from Common import _
-from FileUtilities import children_in_directory
-from Options import options
+    from bleachbit import Windows
 
 # Suppress GTK warning messages while running in CLI #34
 warnings.simplefilter("ignore", Warning)
@@ -51,7 +47,7 @@ warnings.simplefilter("ignore", Warning)
 try:
     import gtk
     HAVE_GTK = True
-except:
+except ImportError:
     HAVE_GTK = False
 
 # a module-level variable for holding cleaners
@@ -96,9 +92,10 @@ class Cleaner:
                 for ds in self.get_deep_scan(option_id):
                     if isinstance(ds, dict):
                         return False
-            except:
-                print 'warning: exception in auto_hide(), cleaner=%s, option=%s' % (self.name, option_id)
-                traceback.print_exc()
+            except Exception:
+                logger = logging.getLogger(__name__)
+                logger.exception('exception in auto_hide(), cleaner=%s, option=%s',
+                                 self.name, option_id)
         return True
 
     def get_commands(self, option_id):
@@ -107,7 +104,7 @@ class Cleaner:
             if option_id == action[0]:
                 for cmd in action[1].get_commands():
                     yield cmd
-        if not self.options.has_key(option_id):
+        if option_id not in self.options:
             raise RuntimeError("Unknown option '%s'" % option_id)
 
     def get_deep_scan(self, option_id):
@@ -116,7 +113,7 @@ class Cleaner:
             if option_id == action[0]:
                 for ds in action[1].get_deep_scan():
                     yield ds
-        if not self.options.has_key(option_id):
+        if option_id not in self.options:
             raise RuntimeError("Unknown option '%s'" % option_id)
 
     def get_description(self):
@@ -152,22 +149,24 @@ class Cleaner:
 
     def is_running(self):
         """Return whether the program is currently running"""
+        logger = logging.getLogger(__name__)
         for running in self.running:
             test = running[0]
             pathname = running[1]
             if 'exe' == test and 'posix' == os.name:
                 if Unix.is_running(pathname):
-                    print "debug: process '%s' is running" % pathname
+                    logger.debug("process '%s' is running", pathname)
                     return True
             elif 'exe' == test and 'nt' == os.name:
                 if Windows.is_process_running(pathname):
-                    print "debug: process '%s' is running" % pathname
+                    logger.debug("process '%s' is running", pathname)
                     return True
             elif 'pathname' == test:
-                expanded = os.path.expanduser(os.path.expandvars(pathname))
+                expanded = expanduser(expandvars(pathname))
                 for globbed in glob.iglob(expanded):
                     if os.path.exists(globbed):
-                        print "debug: file '%s' exists indicating '%s' is running" % (globbed, self.name)
+                        logger.debug(
+                            "file '%s' exists indicating '%s' is running", globbed, self.name)
                         return True
             else:
                 raise RuntimeError(
@@ -222,7 +221,7 @@ class Firefox(Cleaner):
             self.add_running('exe', 'firefox-bin')
             self.add_running('pathname', self.profile_dir + 'lock')
         elif 'nt' == os.name:
-            self.profile_dir = "$USERPROFILE\\Application Data\\Mozilla\\Firefox\\Profiles\\*.default*\\"
+            self.profile_dir = "%APPDATA%\\Mozilla\\Firefox\\Profiles\\*.default*\\"
             self.add_running('exe', 'firefox.exe')
 
         self.description = _("Web browser")
@@ -252,7 +251,7 @@ class Firefox(Cleaner):
             if 'posix' == os.name:
                 # This path is whitelisted under the System - Cache cleaner,
                 # so it can be cleaned here.
-                dirs += [os.path.expanduser('~/.cache/mozilla')]
+                dirs += [expanduser('~/.cache/mozilla')]
             for dirname in dirs:
                 for filename in children_in_directory(dirname, False):
                     yield Command.Delete(filename)
@@ -268,9 +267,9 @@ class Firefox(Cleaner):
                 self.profile_dir, "cookies.sqlite")
         # crash reports
         if 'posix' == os.name:
-            crashdir = os.path.expanduser("~/.mozilla/firefox/Crash Reports")
+            crashdir = expanduser("~/.mozilla/firefox/Crash Reports")
         if 'nt' == os.name:
-            crashdir = os.path.expandvars(
+            crashdir = expandvars(
                 "$USERPROFILE\\Application Data\\Mozilla\\Firefox\\Crash Reports")
         if 'crash_reports' == option_id:
             for filename in children_in_directory(crashdir, False):
@@ -445,7 +444,7 @@ class System(Cleaner):
         # options for Linux and BSD
         #
         if 'posix' == os.name:
-            # TRANSLATORS: desktop entries are .desktop files in Linux tha
+            # TRANSLATORS: desktop entries are .desktop files in Linux that
             # make up the application menu (the menu that shows BleachBit,
             # Firefox, and others.  The .desktop files also associate file
             # types, so clicking on an .html file in Nautilus brings up
@@ -533,7 +532,7 @@ class System(Cleaner):
 
         # cache
         if 'posix' == os.name and 'cache' == option_id:
-            dirname = os.path.expanduser("~/.cache/")
+            dirname = expanduser("~/.cache/")
             for filename in children_in_directory(dirname, True):
                 if self.whitelisted(filename):
                     continue
@@ -620,7 +619,7 @@ class System(Cleaner):
                 '$windir\\system32\\wbem\\Logs\\*.log', )
 
             for path in paths:
-                expanded = os.path.expandvars(path)
+                expanded = expandvars(path)
                 for globbed in glob.iglob(expanded):
                     files += [globbed]
 
@@ -632,15 +631,15 @@ class System(Cleaner):
         # how to manually create this file
         # http://www.pctools.com/guides/registry/detail/856/
         if 'nt' == os.name and 'memory_dump' == option_id:
-            fname = os.path.expandvars('$windir\\memory.dmp')
+            fname = expandvars('$windir\\memory.dmp')
             if os.path.exists(fname):
                 files += [fname]
-            for fname in glob.iglob(os.path.expandvars('$windir\\Minidump\\*.dmp')):
+            for fname in glob.iglob(expandvars('$windir\\Minidump\\*.dmp')):
                 files += [fname]
 
         # most recently used documents list
         if 'posix' == os.name and 'recent_documents' == option_id:
-            files += [os.path.expanduser("~/.recently-used")]
+            files += [expanduser("~/.recently-used")]
             # GNOME 2.26 (as seen on Ubuntu 9.04) will retain the list
             # in memory if it is simply deleted, so it must be shredded
             # (or at least truncated).
@@ -650,12 +649,19 @@ class System(Cleaner):
             # shredding.
             #
             # https://bugzilla.gnome.org/show_bug.cgi?id=591404
+
+            def gtk_purge_items():
+                """Purge GTK items"""
+                gtk.RecentManager().purge_items()
+                yield 0
+
             for pathname in ["~/.recently-used.xbel", "~/.local/share/recently-used.xbel"]:
-                pathname = os.path.expanduser(pathname)
+                pathname = expanduser(pathname)
                 if os.path.lexists(pathname):
                     yield Command.Shred(pathname)
-                    if HAVE_GTK:
-                        gtk.RecentManager().purge_items()
+            if HAVE_GTK:
+                # Use the Function to skip when in preview mode
+                yield Command.Function(None, gtk_purge_items, _('Recent documents list'))
 
         if 'posix' == os.name and 'rotated_logs' == option_id:
             for path in Unix.rotated_logs():
@@ -676,21 +682,29 @@ class System(Cleaner):
 
         # temporary files
         if 'nt' == os.name and 'tmp' == option_id:
-            dirname = os.path.expandvars(
+            dirname1 = expandvars(
                 "$USERPROFILE\\Local Settings\\Temp\\")
+            dirname2 = expandvars(r'%temp%')
+            dirname3 = expandvars("%windir%\\temp\\")
+            dirnames = []
+            if Windows.get_windows_version() >= 6.0:
+                # Windows Vista or later
+                dirnames.append(dirname2)
+            else:
+                # Windows XP
+                dirnames.append(dirname1)
+            dirnames.append(dirname3)
             # whitelist the folder %TEMP%\Low but not its contents
             # https://bugs.launchpad.net/bleachbit/+bug/1421726
-            low = os.path.join(dirname, 'low').lower()
-            for filename in children_in_directory(dirname, True):
-                if not low == filename.lower():
-                    yield Command.Delete(filename)
-            dirname = os.path.expandvars("$windir\\temp\\")
-            for filename in children_in_directory(dirname, True):
-                yield Command.Delete(filename)
+            for dirname in dirnames:
+                low = os.path.join(dirname, 'low').lower()
+                for filename in children_in_directory(dirname, True):
+                    if not low == filename.lower():
+                        yield Command.Delete(filename)
 
         # trash
         if 'posix' == os.name and 'trash' == option_id:
-            dirname = os.path.expanduser("~/.Trash")
+            dirname = expanduser("~/.Trash")
             for filename in children_in_directory(dirname, False):
                 yield Command.Delete(filename)
             # fixme http://www.ramendik.ru/docs/trashspec.html
@@ -698,13 +712,13 @@ class System(Cleaner):
             # ~/.local/share/Trash
             # * GNOME 2.22, Fedora 9
             # * KDE 4.1.3, Ubuntu 8.10
-            dirname = os.path.expanduser("~/.local/share/Trash/files")
+            dirname = expanduser("~/.local/share/Trash/files")
             for filename in children_in_directory(dirname, True):
                 yield Command.Delete(filename)
-            dirname = os.path.expanduser("~/.local/share/Trash/info")
+            dirname = expanduser("~/.local/share/Trash/info")
             for filename in children_in_directory(dirname, True):
                 yield Command.Delete(filename)
-            dirname = os.path.expanduser("~/.local/share/Trash/expunged")
+            dirname = expanduser("~/.local/share/Trash/expunged")
             # desrt@irc.gimpnet.org tells me that the trash
             # backend puts files in here temporary, but in some situations
             # the files are stuck.
@@ -747,13 +761,15 @@ class System(Cleaner):
 
         # prefetch
         if 'nt' == os.name and 'prefetch' == option_id:
-            for path in glob.iglob(os.path.expandvars('$windir\\Prefetch\\*.pf')):
+            for path in glob.iglob(expandvars('$windir\\Prefetch\\*.pf')):
                 yield Command.Delete(path)
 
         # recycle bin
         if 'nt' == os.name and 'recycle_bin' == option_id:
             # This method allows shredding
+            recycled_any = False
             for path in Windows.get_recycle_bin():
+                recycled_any = True
                 yield Command.Delete(path)
             # If there were any files deleted, Windows XP will show the
             # wrong icon for the recycle bin indicating it is not empty.
@@ -765,14 +781,19 @@ class System(Cleaner):
             # opens the recycle bin folder.
 
             # This is a hack to refresh the icon.
-            import tempfile
-            tmpdir = tempfile.mkdtemp()
-            Windows.move_to_recycle_bin(tmpdir)
-            try:
-                Windows.empty_recycle_bin(None, True)
-            except:
-                logger = logging.getLogger(__name__)
-                logger.info('error in empty_recycle_bin()', exc_info=True)
+            def empty_recycle_bin_func():
+                import tempfile
+                tmpdir = tempfile.mkdtemp()
+                Windows.move_to_recycle_bin(tmpdir)
+                try:
+                    Windows.empty_recycle_bin(None, True)
+                except:
+                    logging.getLogger(__name__).info('error in empty_recycle_bin()', exc_info=True)
+                yield 0
+            # Using the Function Command prevents emptying the recycle bin
+            # when in preview mode.
+            if recycled_any:
+                yield Command.Function(None, empty_recycle_bin_func, _('Empty the recycle bin'))
 
         # Windows Updates
         if 'nt' == os.name and 'updates' == option_id:
@@ -799,14 +820,20 @@ class System(Cleaner):
             '^/tmp/orbit-[^/]+/bonobo-activation-register[a-z0-9-]*.lock$',
             '^/tmp/orbit-[^/]+/bonobo-activation-server-[a-z0-9-]*ior$',
             '^/tmp/pulse-[^/]+/pid$',
-            '^/var/tmp/kdecache-']
-        regexes.append('^' + os.path.expanduser('~/.cache/wallpaper/'))
-        # Clean Firefox cache from Firefox cleaner (LP#1295826)
-        regexes.append('^' + os.path.expanduser('~/.cache/mozilla'))
-        regexes.append(
-            '^' + os.path.expanduser('~/.cache/gnome-control-center/'))
+            '^/var/tmp/kdecache-',
+            '^' + expanduser('~/.cache/wallpaper/'),
+            # Clean Firefox cache from Firefox cleaner (LP#1295826)
+            '^' + expanduser('~/.cache/mozilla'),
+            # Clean Google Chrome cache from Google Chrome cleaner (LP#656104)
+            '^' + expanduser('~/.cache/google-chrome'),
+            '^' + expanduser('~/.cache/gnome-control-center/'),
+            # iBus Pinyin
+            # https://bugs.launchpad.net/bleachbit/+bug/1538919
+            '^' + expanduser('~/.cache/ibus/'),
+            # Linux Bluetooth daemon obexd
+            '^' + expanduser('~/.cache/obexd/')]
         for regex in regexes:
-            if None != re.match(regex, pathname):
+            if re.match(regex, pathname) is not None:
                 return True
         return False
 
@@ -825,12 +852,12 @@ def register_cleaners():
     backends["system"] = System()
 
     # register CleanerML cleaners
-    import CleanerML
+    from bleachbit import CleanerML
     CleanerML.load_cleaners()
 
     # register Winapp2.ini cleaners
     if 'nt' == os.name:
-        import Winapp
+        from bleachbit import Winapp
         Winapp.load_cleaners()
 
 
@@ -840,7 +867,7 @@ def create_simple_cleaner(paths):
     cleaner.add_option(option_id='files', name='', description='')
     cleaner.name = _("System")  # shows up in progress bar
 
-    import Action
+    from bleachbit import Action
 
     class CustomFileAction(Action.ActionProvider):
         action_key = '__customfileaction'
@@ -878,7 +905,7 @@ def create_wipe_cleaner(path):
             yield ret
         yield 0
 
-    import Action
+    from bleachbit import Action
 
     class CustomWipeAction(Action.ActionProvider):
         action_key = '__customwipeaction'
