@@ -1,8 +1,8 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2016 Andrew Ziem
-# http://www.bleachbit.org
+# Copyright (C) 2008-2018 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,19 +22,21 @@
 Create cleaners from CleanerML (markup language)
 """
 
+from __future__ import absolute_import, print_function
+
+import bleachbit
+from bleachbit.Action import ActionProvider
+from bleachbit import _
+from bleachbit.General import boolstr_to_bool, getText
+from bleachbit.FileUtilities import listdir
+from bleachbit import Cleaner
 
 import logging
 import os
 import sys
-import traceback
 import xml.dom.minidom
-import Cleaner
-import Common
 
-from Action import ActionProvider
-from Common import _
-from General import boolstr_to_bool, getText
-from FileUtilities import listdir
+logger = logging.getLogger(__name__)
 
 
 class CleanerML:
@@ -55,7 +57,7 @@ class CleanerML:
         self.option_description = None
         self.option_warning = None
         self.xlate_cb = xlate_cb
-        if None == self.xlate_cb:
+        if self.xlate_cb is None:
             self.xlate_mode = False
             self.xlate_cb = lambda x, y=None: None  # do nothing
         else:
@@ -94,10 +96,8 @@ class CleanerML:
             try:
                 self.handle_cleaner_option(option)
             except:
-                logger = logging.getLogger(__name__)
-                logger.exception(
-                    'error in handle_cleaner_option() for cleaner id = %s, option XML=%s',
-                    self.cleaner.id, option.toxml())
+                logger.exception('error in handle_cleaner_option() for cleaner id = %s, option XML=%s',
+                                 self.cleaner.id, option.toxml())
         self.handle_cleaner_running(cleaner.getElementsByTagName('running'))
         self.handle_localizations(
             cleaner.getElementsByTagName('localizations'))
@@ -171,7 +171,7 @@ class CleanerML:
         for actionplugin in ActionProvider.plugins:
             if actionplugin.action_key == command:
                 provider = actionplugin(action_node)
-        if None == provider:
+        if provider is None:
             raise RuntimeError("Invalid command '%s'" % command)
         self.cleaner.add_action(self.option_id, provider)
 
@@ -179,7 +179,7 @@ class CleanerML:
         """<localizations> element under <cleaner>"""
         if not 'posix' == os.name:
             return
-        import Unix
+        from bleachbit import Unix
         for localization_node in localization_nodes:
             for child_node in localization_node.childNodes:
                 Unix.locales.add_xml(child_node)
@@ -189,42 +189,41 @@ class CleanerML:
 
 def list_cleanerml_files(local_only=False):
     """List CleanerML files"""
-    cleanerdirs = (Common.local_cleaners_dir,
-                   Common.personal_cleaners_dir)
-    if not local_only and Common.system_cleaners_dir:
-        cleanerdirs += (Common.system_cleaners_dir, )
+    cleanerdirs = (bleachbit.personal_cleaners_dir, )
+    if bleachbit.local_cleaners_dir:
+        # If the application is installed, locale_cleaners_dir is None
+        cleanerdirs = (bleachbit.local_cleaners_dir, )
+    if not local_only and bleachbit.system_cleaners_dir:
+        cleanerdirs += (bleachbit.system_cleaners_dir, )
     for pathname in listdir(cleanerdirs):
         if not pathname.lower().endswith('.xml'):
             continue
         import stat
         st = os.stat(pathname)
         if sys.platform != 'win32' and stat.S_IMODE(st[stat.ST_MODE]) & 2:
-            logger = logging.getLogger(__name__)
-            logger.warning(
-                "ignoring cleaner because it is world writable: %s" % pathname)
+            logger.warning("ignoring cleaner because it is world writable: %s", pathname)
             continue
         yield pathname
 
 
 def load_cleaners():
     """Scan for CleanerML and load them"""
-    logger = logging.getLogger(__name__)
     for pathname in list_cleanerml_files():
         try:
             xmlcleaner = CleanerML(pathname)
         except:
-            logger.exception('error reading cleaner: %s' % pathname)
+            logger.exception('error reading cleaner: %s', pathname)
             continue
         cleaner = xmlcleaner.get_cleaner()
         if cleaner.is_usable():
             Cleaner.backends[cleaner.id] = cleaner
         else:
-            logger.debug(
-                'cleaner is not usable on this OS because it has no actions: %s' % pathname)
+            logger.debug('cleaner is not usable on this OS because it has no actions: %s', pathname)
 
 
 def pot_fragment(msgid, pathname, translators=None):
     """Create a string fragment for generating .pot files"""
+    msgid = msgid.replace('"', '\\"') # escape quotation mark
     if translators:
         translators = "#. %s\n" % translators
     else:
@@ -251,15 +250,9 @@ def create_pot():
                       lambda newstr, translators=None:
                       strings.append([newstr, translators]))
         except:
-            logger = logging.getLogger(__name__)
-            logger.exception('error reading: %s' % pathname)
+            logger.exception('error reading: %s', pathname)
             continue
         for (string, translators) in strings:
             f.write(pot_fragment(string, pathname, translators))
 
     f.close()
-
-
-if __name__ == '__main__':
-    if 2 == len(sys.argv) and 'pot' == sys.argv[1]:
-        create_pot()
