@@ -1,8 +1,8 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2015 Andrew Ziem
-# http://bleachbit.sourceforge.net
+# Copyright (C) 2008-2018 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,18 +22,21 @@
 Command design pattern implementation for cleaning
 """
 
+from __future__ import absolute_import, print_function
 
+from bleachbit import _
+from bleachbit import FileUtilities
+
+import logging
 import os
 import types
-import FileUtilities
 
 from sqlite3 import DatabaseError
-from Common import _
 
 if 'nt' == os.name:
-    import Windows
+    import bleachbit.Windows
 else:
-    from General import WindowsError
+    from bleachbit.General import WindowsError
 
 
 def whitelist(path):
@@ -59,6 +62,10 @@ class Delete:
         self.path = path
         self.shred = False
 
+    def __str__(self):
+        return 'Command to %s %s' % \
+            ('shred' if self.shred else 'delete', self.path)
+
     def execute(self, really_delete):
         """Make changes and return results"""
         if FileUtilities.whitelisted(self.path):
@@ -75,17 +82,21 @@ class Delete:
         if really_delete:
             try:
                 FileUtilities.delete(self.path, self.shred)
-            except WindowsError, e:
+            except WindowsError as e:
                 # WindowsError: [Error 32] The process cannot access the file because it is being
                 # used by another process: u'C:\\Documents and
                 # Settings\\username\\Cookies\\index.dat'
                 if 32 != e.winerror and 5 != e.winerror:
                     raise
                 try:
-                    Windows.delete_locked_file(self.path)
+                    bleachbit.Windows.delete_locked_file(self.path)
                 except:
                     raise
                 else:
+                    if self.shred:
+                        import warnings
+                        warnings.warn(
+                            _('At least one file was locked by another process, so its contents could not be overwritten. It will be marked for deletion upon system reboot.'))
                     # TRANSLATORS: The file will be deleted when the
                     # system reboots
                     ret['label'] = _('Mark for deletion')
@@ -104,13 +115,13 @@ class Function:
         self.func = func
         self.label = label
         try:
-            assert(isinstance(func, types.FunctionType))
+            assert isinstance(func, types.FunctionType)
         except AssertionError:
             raise AssertionError('Expected MethodType but got %s' % type(func))
 
     def execute(self, really_delete):
 
-        if None != self.path and FileUtilities.whitelisted(self.path):
+        if self.path is not None and FileUtilities.whitelisted(self.path):
             yield whitelist(self.path)
             return
 
@@ -122,7 +133,7 @@ class Function:
             'size': None}
 
         if really_delete:
-            if None == self.path:
+            if self.path is None:
                 # Function takes no path.  It returns the size.
                 func_ret = self.func()
                 if isinstance(func_ret, types.GeneratorType):
@@ -133,22 +144,25 @@ class Function:
                             # If tuple, then display progress.
                             yield func_ret
                 # either way, func_ret should be an integer
-                assert(isinstance(func_ret, (int, long)))
+                assert isinstance(func_ret, (int, long))
                 ret['size'] = func_ret
             else:
+                if os.path.isdir(self.path):
+                    raise RuntimeError('Attempting to run file function %s on directory %s' %
+                                       (self.func.func_name, self.path))
                 # Function takes a path.  We check the size.
                 oldsize = FileUtilities.getsize(self.path)
                 try:
                     self.func(self.path)
-                except DatabaseError, e:
+                except DatabaseError as e:
                     if -1 == e.message.find('file is encrypted or is not a database') and \
                        -1 == e.message.find('or missing database'):
                         raise
-                    print 'Warning:', e.message
+                    logging.getLogger(__name__).exception(e.message)
                     return
                 try:
                     newsize = FileUtilities.getsize(self.path)
-                except OSError, e:
+                except OSError as e:
                     from errno import ENOENT
                     if e.errno == ENOENT:
                         # file does not exist
@@ -169,6 +183,10 @@ class Ini:
         self.section = section
         self.parameter = parameter
 
+    def __str__(self):
+        return 'Command to clean .ini path=%s, section=-%s, parameter=%s ' % \
+            (self.path, self.section, self.parameter)
+
     def execute(self, really_delete):
         """Make changes and return results"""
 
@@ -200,6 +218,10 @@ class Json:
         self.path = path
         self.address = address
 
+    def __str(self):
+        return 'Command to clean JSON file, path=%s, address=%s ' % \
+            (self.path, self.address)
+
     def execute(self, really_delete):
         """Make changes and return results"""
 
@@ -230,11 +252,17 @@ class Shred(Delete):
         Delete.__init__(self, path)
         self.shred = True
 
+    def __str__(self):
+        return 'Command to shred %s' % self.path
+
 
 class Truncate(Delete):
 
     """Truncate a single file"""
 
+    def __str__(self):
+        return 'Command to truncate %s' % self.path
+
     def execute(self, really_delete):
         """Make changes and return results"""
 
@@ -264,6 +292,9 @@ class Winreg:
         self.keyname = keyname
         self.valuename = valuename
 
+    def __str__(self):
+        return 'Command to clean registry, key=%s, value=%s ' % (self.keyname, self.valuename)
+
     def execute(self, really_delete):
         """Execute the Windows registry cleaner"""
         if 'nt' != os.name:
@@ -272,10 +303,10 @@ class Winreg:
         ret = None  # return value meaning 'deleted' or 'delete-able'
         if self.valuename:
             _str = '%s<%s>' % (self.keyname, self.valuename)
-            ret = Windows.delete_registry_value(self.keyname,
+            ret = bleachbit.Windows.delete_registry_value(self.keyname,
                                                 self.valuename, really_delete)
         else:
-            ret = Windows.delete_registry_key(self.keyname, really_delete)
+            ret = bleachbit.Windows.delete_registry_key(self.keyname, really_delete)
             _str = self.keyname
         if not ret:
             # Nothing to delete or nothing was deleted.  This return
