@@ -2,8 +2,8 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2014 Andrew Ziem
-# http://bleachbit.sourceforge.net
+# Copyright (C) 2008-2018 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,9 +23,41 @@
 Scan directory tree for files to delete
 """
 
+from __future__ import absolute_import, print_function
 
+import logging
 import os
+import platform
 import re
+import unicodedata
+
+
+UTF8 = 'utf-8'
+
+
+def to_unicode(s):
+    """
+    Converts non-unicode UTF-8 string to unicode obj. Does nothing if
+    string is already unicode.
+    """
+    return s if isinstance(s, unicode) else unicode(s, UTF8)
+
+
+def normalized_walk(top, **kwargs):
+    """
+    macOS uses decomposed UTF-8 to store filenames. This functions
+    is like `os.walk` but recomposes those decomposed filenames on
+    macOS
+    """
+    if 'Darwin' == platform.system():
+        for dirpath, dirnames, filenames in os.walk(top, **kwargs):
+            yield dirpath, dirnames, [
+                unicodedata.normalize('NFC', to_unicode(fn)).encode(UTF8)
+                for fn in filenames
+            ]
+    else:
+        for result in os.walk(top, **kwargs):
+            yield result
 
 
 class DeepScan:
@@ -38,19 +70,19 @@ class DeepScan:
 
     def add_search(self, dirname, regex):
         """Starting in dirname, look for files matching regex"""
-        if not self.searches.has_key(dirname):
+        if dirname not in self.searches:
             self.searches[dirname] = [regex]
         else:
             self.searches[dirname].append(regex)
 
     def scan(self):
         """Perform requested searches and yield each match"""
-        print 'debug: DeepScan.scan: searches=', self.searches
+        logging.getLogger(__name__).debug('DeepScan.scan: searches=%s', str(self.searches))
         import time
         yield_time = time.time()
 
-        for (top, regexes) in self.searches.iteritems():
-            for (dirpath, dirnames, filenames) in os.walk(top):
+        for (top, regexes) in self.searches.items():
+            for (dirpath, dirnames, filenames) in normalized_walk(top):
                 for regex in regexes:
                     # fixme, don't match filename twice
                     r = re.compile(regex)
