@@ -1,8 +1,8 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2015 Andrew Ziem
-# http://bleachbit.sourceforge.net
+# Copyright (C) 2008-2018 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,16 +17,21 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-
 """
 General code
 """
 
+from __future__ import absolute_import, print_function
 
+import bleachbit
+
+import logging
 import os
 import sys
 import traceback
 
+logger = logging.getLogger(__name__)
+
 
 #
 # XML
@@ -68,9 +73,9 @@ def chownself(path):
     if 'posix' != os.name:
         return
     uid = getrealuid()
-    print 'debug: chown(%s, uid=%s)' % (path, uid)
+    logger.debug('chown(%s, uid=%s)', path, uid)
     if 0 == path.find('/root'):
-        print 'note: chown for path /root aborted'
+        logger.info('chown for path /root aborted')
         return
     try:
         os.chown(path, uid, -1)
@@ -87,8 +92,6 @@ def getrealuid():
     if os.getenv('SUDO_UID'):
         return int(os.getenv('SUDO_UID'))
 
-    login = None
-
     try:
         login = os.getlogin()
         # On Ubuntu 9.04, getlogin() under sudo returns non-root user.
@@ -107,42 +110,52 @@ def getrealuid():
 def makedirs(path):
     """Make directory recursively considering sudo permissions.
     'Path' should not end in a delimiter."""
-    print "debug: makedirs(%s)" % path
+    logger.debug('makedirs(%s)', path.encode(bleachbit.FSE))
     if os.path.lexists(path):
         return
     parentdir = os.path.split(path)[0]
     if not os.path.lexists(parentdir):
         makedirs(parentdir)
-    os.mkdir(path, 0700)
+    os.mkdir(path, 0o700)
     if sudo_mode():
         chownself(path)
 
 
-def run_external(args, stdout=False, env=None):
+def run_external(args, stdout=False, env=None, clean_env=True):
     """Run external command and return (return code, stdout, stderr)"""
-    print 'debug: running cmd ', args
+    logger.debug('running cmd ' + ' '.join(args))
     import subprocess
-    if False == stdout:
+    if not stdout:
         stdout = subprocess.PIPE
     kwargs = {}
     if subprocess.mswindows:
         # hide the 'DOS box' window
+        import win32process, win32con
         stui = subprocess.STARTUPINFO()
-        import win32process
         stui.dwFlags = win32process.STARTF_USESHOWWINDOW
-        import win32con
         stui.wShowWindow = win32con.SW_HIDE
         kwargs['startupinfo'] = stui
+    if not env and clean_env and 'posix' == os.name:
+        # Clean environment variables so that that subprocesses use English
+        # instead of translated text. This helps when checking for certain
+        # strings in the output.
+        # https://github.com/bleachbit/bleachbit/issues/167
+        # https://github.com/bleachbit/bleachbit/issues/168
+        keep_env = ('PATH', 'HOME', 'LD_LIBRARY_PATH', 'TMPDIR')
+        env = dict((key, value)
+                   for key, value in os.environ.iteritems() if key in keep_env)
+        env['LANG'] = 'C'
+        env['LC_ALL'] = 'C'
     p = subprocess.Popen(args, stdout=stdout,
                          stderr=subprocess.PIPE, env=env, **kwargs)
     try:
         out = p.communicate()
     except KeyboardInterrupt:
         out = p.communicate()
-        print out[0]
-        print out[1]
+        print(out[0])
+        print(out[1])
         raise
-    return (p.returncode, out[0], out[1])
+    return p.returncode, out[0], out[1]
 
 
 def sudo_mode():
@@ -156,4 +169,4 @@ def sudo_mode():
         # Fedora 13: os.getenv('USER') = 'root' under sudo
         # return False
 
-    return os.getenv('SUDO_UID') != None
+    return os.getenv('SUDO_UID') is not None
