@@ -1,8 +1,9 @@
+
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2015 Andrew Ziem
-# http://bleachbit.sourceforge.net
+# Copyright (C) 2008-2018 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,20 +23,20 @@
 Import Winapp2.ini files
 """
 
+from __future__ import absolute_import, print_function
 
-import Cleaner
-import Common
-import ConfigParser
-import Windows
+import bleachbit
+from bleachbit import Cleaner, Windows
+from bleachbit.Action import Delete, Winreg
+from bleachbit import _, FSE, expandvars
+
+import logging
 import os
 import glob
 import re
-import traceback
-
-from Action import Delete, Winreg
-from Common import _
 from xml.dom.minidom import parseString
 
+logger = logging.getLogger(__name__)
 
 MAX_DETECT = 50
 
@@ -99,29 +100,38 @@ def detectos(required_ver, mock=False):
 
 def winapp_expand_vars(pathname):
     """Expand environment variables using special Winapp2.ini rules"""
-    # Change %foo% to ${foo} as required by Python 2.5.4 (but not 2.7.8)
-    pathname = re.sub(r'%([a-zA-Z0-9]+)%', r'${\1}', pathname)
     # This is the regular expansion
-    expand1 = os.path.expandvars(pathname)
+    expand1 = expandvars(pathname)
     # Winapp2.ini expands %ProgramFiles% to %ProgramW6432%, etc.
     subs = (('ProgramFiles', 'ProgramW6432'),
             ('CommonProgramFiles', 'CommonProgramW6432'))
     for (sub_orig, sub_repl) in subs:
-        pattern = re.compile(r'\${%s}' % sub_orig, flags=re.IGNORECASE)
+        pattern = re.compile('%{}%'.format(sub_orig), flags=re.IGNORECASE)
         if pattern.match(pathname):
-            expand2 = pattern.sub('${%s}' % sub_repl, pathname)
-            return (expand1, os.path.expandvars(expand2))
-    return (expand1,)
+            expand2 = pattern.sub('%{}%'.format(sub_repl), pathname)
+            return expand1, expandvars(expand2)
+    return expand1,
 
 
 def detect_file(pathname):
     """Check whether a path exists for DetectFile#="""
     for expanded in winapp_expand_vars(pathname):
-        for thispath in glob.iglob(expanded):
+        for _ in glob.iglob(expanded):
             return True
     return False
 
 
+def fnmatch_translate(pattern):
+    """Same as the original without the end"""
+    import fnmatch
+    r = fnmatch.translate(pattern)
+    if r.endswith('$'):
+        return r[:-1]
+    if r.endswith(r'\Z(?ms)'):
+        return r[:-7]
+    return r
+
+
 class Winapp:
 
     """Create cleaners from a Winapp2.ini-style file"""
@@ -134,15 +144,14 @@ class Winapp:
         for langsecref in set(langsecref_map.values()):
             self.add_section(langsecref[0], langsecref[1])
         self.errors = 0
-        self.parser = ConfigParser.RawConfigParser()
+        self.parser = bleachbit.RawConfigParser()
         self.parser.read(pathname)
         for section in self.parser.sections():
             try:
                 self.handle_section(section)
-            except:
+            except Exception:
                 self.errors += 1
-                print 'ERROR: parsing error in section %s' % section
-                traceback.print_exc()
+                logger.exception('parsing error in section %s', section)
 
     def add_section(self, cleaner_id, name):
         """Add a section (cleaners)"""
@@ -160,23 +169,74 @@ class Winapp:
             return langsecref_map[langsecref][0]
         # custom, such as games
         cleanerid = 'winapp2_' + section2option(langsecref)
-        if not cleanerid in self.cleaners:
+        if cleanerid not in self.cleaners:
             # never seen before
             self.add_section(cleanerid, langsecref)
         return cleanerid
 
+    def excludekey_to_nwholeregex(self, excludekey):
+        """Translate one ExcludeKey to CleanerML nwholeregex
+
+        Supported examples
+        FILE=%LocalAppData%\BleachBit\BleachBit.ini
+        FILE=%LocalAppData%\BleachBit\|BleachBit.ini
+        FILE=%LocalAppData%\BleachBit\|*.ini
+        FILE=%LocalAppData%\BleachBit\|*.ini;*.bak
+        PATH=%LocalAppData%\BleachBit\
+        PATH=%LocalAppData%\BleachBit\|*.*
+        """
+        parts = excludekey.split('|')
+        parts[0] = parts[0].upper()
+        if 'REG' == parts[0]:
+            raise NotImplementedError('REG not supported in ExcludeKey')
+
+        # the last part contains the filename(s)
+        files = None
+        files_regex = ''
+        if 3 == len(parts):
+            files = parts[2].split(';')
+            if 1 == len(files):
+                # one file pattern like *.* or *.log
+                files_regex = fnmatch_translate(files[0])
+                if '*.*' == files_regex:
+                    files = None
+            elif len(files) > 1:
+                # multiple file patterns like *.log;*.bak
+                files_regex = '(%s)' % '|'.join(
+                    [fnmatch_translate(f) for f in files])
+
+        # the middle part contains the file
+        regexes = []
+        for expanded in winapp_expand_vars(parts[1]):
+            regex = None
+            if not files:
+                # There is no third part, so this is either just a folder,
+                # or sometimes the file is specified directly.
+                regex = fnmatch_translate(expanded)
+            if files:
+                # match one or more file types, directly in this tree or in any
+                # sub folder
+                regex = '%s.*%s' % (
+                    fnmatch_translate(expanded), files_regex)
+            regexes.append(regex)
+
+        if 1 == len(regexes):
+            return regexes[0]
+        else:
+            return '(%s)' % '|'.join(regexes)
+
     def handle_section(self, section):
         """Parse a section"""
         # if simple detection fails then discard the section
         if self.parser.has_option(section, 'detect'):
-            key = self.parser.get(section, 'detect')
+            key = self.parser.get(section, 'detect').decode(FSE)
             if not Windows.detect_registry_key(key):
                 return
         if self.parser.has_option(section, 'detectfile'):
-            if not detect_file(self.parser.get(section, 'detectfile')):
+            if not detect_file(self.parser.get(section, 'detectfile').decode(FSE)):
                 return
         if self.parser.has_option(section, 'detectos'):
-            required_ver = self.parser.get(section, 'detectos')
+            required_ver = self.parser.get(section, 'detectos').decode(FSE)
             if not detectos(required_ver):
                 return
         # in case of multiple detection, discard if none match
@@ -185,8 +245,8 @@ class Winapp:
             for n in range(1, MAX_DETECT):
                 option_id = 'detectfile%d' % n
                 if self.parser.has_option(section, option_id):
-                    if detect_file(self.parser.get(section, option_id)):
-                        matches = matches + 1
+                    if detect_file(self.parser.get(section, option_id).decode(FSE)):
+                        matches += 1
             if 0 == matches:
                 return
         if self.parser.has_option(section, 'detect1'):
@@ -194,23 +254,27 @@ class Winapp:
             for n in range(1, MAX_DETECT):
                 option_id = 'detect%d' % n
                 if self.parser.has_option(section, option_id):
-                    if Windows.detect_registry_key(self.parser.get(section, option_id)):
-                        matches = matches + 1
+                    if Windows.detect_registry_key(self.parser.get(section, option_id).decode(FSE)):
+                        matches += 1
             if 0 == matches:
                 return
-        # not yet implemented
-        if self.parser.has_option(section, 'excludekey'):
-            print 'ERROR: ExcludeKey not implemented, section=', section
-            return
+        # excludekeys ignores a file, path, or registry key
+        excludekeys = []
+        if self.parser.has_option(section, 'excludekey1'):
+            for n in range(1, MAX_DETECT):
+                option_id = 'excludekey%d' % n
+                if self.parser.has_option(section, option_id):
+                    excludekeys.append(
+                        self.excludekey_to_nwholeregex(self.parser.get(section, option_id).decode(FSE)))
         # there are two ways to specify sections: langsecref= and section=
         if self.parser.has_option(section, 'langsecref'):
             # verify the langsecref number is known
             # langsecref_num is 3021, games, etc.
-            langsecref_num = self.parser.get(section, 'langsecref')
+            langsecref_num = self.parser.get(section, 'langsecref').decode(FSE)
         elif self.parser.has_option(section, 'section'):
-            langsecref_num = self.parser.get(section, 'section')
+            langsecref_num = self.parser.get(section, 'section').decode(FSE)
         else:
-            print 'ERROR: neither option LangSecRef nor Section found in section %s' % (section)
+            logger.error('neither option LangSecRef nor Section found in section %s', section)
             return
         # find the BleachBit internal cleaner ID
         lid = self.section_to_cleanerid(langsecref_num)
@@ -218,21 +282,21 @@ class Winapp:
             section2option(section), section.replace('*', ''), '')
         for option in self.parser.options(section):
             if option.startswith('filekey'):
-                self.handle_filekey(lid, section, option)
+                self.handle_filekey(lid, section, option, excludekeys)
             elif option.startswith('regkey'):
                 self.handle_regkey(lid, section, option)
             elif option == 'warning':
                 self.cleaners[lid].set_warning(
-                    section2option(section), self.parser.get(section, 'warning'))
+                    section2option(section), self.parser.get(section, 'warning').decode(FSE))
             elif option in ('default', 'detectfile', 'detect', 'langsecref', 'section') \
                 or ['detect%d' % x for x in range(1, MAX_DETECT)] \
                     or ['detectfile%d' % x for x in range(1, MAX_DETECT)]:
                 pass
             else:
-                print 'WARNING: unknown option %s in section %s' % (option, section)
+                logger.warning('unknown option %s in section %s', option, section)
                 return
 
-    def __make_file_provider(self, dirname, filename, recurse, removeself):
+    def __make_file_provider(self, dirname, filename, recurse, removeself, excludekeys):
         """Change parsed FileKey to action provider"""
         regex = ''
         if recurse:
@@ -244,25 +308,35 @@ class Winapp:
                 if removeself:
                     search = 'walk.all'
             else:
-                regex = ' regex="%s" ' % (re.escape(filename) + '$')
+                import fnmatch
+                regex = ' regex="%s" ' % (fnmatch.translate(filename))
         else:
             search = 'glob'
             path = os.path.join(dirname, filename)
             if -1 == path.find('*'):
                 search = 'file'
-        action_str = '<option command="delete" search="%s" path="%s" %s/>' % \
-            (search, xml_escape(path), regex)
+        excludekeysxml = ''
+        if excludekeys:
+            if len(excludekeys) > 1:
+                # multiple
+                exclude_str = '(%s)' % '|'.join(excludekeys)
+            else:
+                # just one
+                exclude_str = excludekeys[0]
+            excludekeysxml = 'nwholeregex="%s"' % exclude_str
+        action_str = u'<option command="delete" search="%s" path="%s" %s %s/>' % \
+                     (search, xml_escape(path), regex, excludekeysxml)
         yield Delete(parseString(action_str).childNodes[0])
         if removeself:
-            action_str = '<option command="delete" search="file" path="%s"/>' % xml_escape(
-                dirname)
+            action_str = u'<option command="delete" search="file" path="%s"/>' % \
+                         (xml_escape(dirname))
             yield Delete(parseString(action_str).childNodes[0])
 
-    def handle_filekey(self, lid, ini_section, ini_option):
+    def handle_filekey(self, lid, ini_section, ini_option, excludekeys):
         """Parse a FileKey# option.
 
         Section is [Application Name] and option is the FileKey#"""
-        elements = self.parser.get(ini_section, ini_option).strip().split('|')
+        elements = self.parser.get(ini_section, ini_option).decode(FSE).strip().split('|')
         dirnames = winapp_expand_vars(elements.pop(0))
         filenames = ""
         if elements:
@@ -277,16 +351,16 @@ class Winapp:
                 recurse = True
                 removeself = True
             else:
-                print 'WARNING: unknown file option %s in section %s' % (element, ini_section)
+                logger.warning('unknown file option %s in section %s', element, ini_section)
         for filename in filenames.split(';'):
             for dirname in dirnames:
-                for provider in self.__make_file_provider(dirname, filename, recurse, removeself):
+                for provider in self.__make_file_provider(dirname, filename, recurse, removeself, excludekeys):
                     self.cleaners[lid].add_action(
                         section2option(ini_section), provider)
 
     def handle_regkey(self, lid, ini_section, ini_option):
         """Parse a RegKey# option"""
-        elements = self.parser.get(ini_section, ini_option).strip().split('|')
+        elements = self.parser.get(ini_section, ini_option).decode(FSE).strip().split('|')
         path = xml_escape(elements[0])
         name = ""
         if 2 == len(elements):
@@ -304,7 +378,7 @@ class Winapp:
 
 def list_winapp_files():
     """List winapp2.ini files"""
-    for dirname in (Common.personal_cleaners_dir, Common.system_cleaners_dir):
+    for dirname in (bleachbit.personal_cleaners_dir, bleachbit.system_cleaners_dir):
         fn = os.path.join(dirname, 'winapp2.ini')
         if os.path.exists(fn):
             yield fn
@@ -315,9 +389,8 @@ def load_cleaners():
     for pathname in list_winapp_files():
         try:
             inicleaner = Winapp(pathname)
-        except:
-            print "Error reading winapp2.ini cleaner '%s'" % pathname
-            traceback.print_exc()
+        except Exception as e:
+            logger.exception("Error reading winapp2.ini cleaner '%s'", pathname)
         else:
             for cleaner in inicleaner.get_cleaners():
                 Cleaner.backends[cleaner.id] = cleaner
