@@ -1,8 +1,8 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2015 Andrew Ziem
-# http://bleachbit.sourceforge.net
+# Copyright (C) 2008-2018 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,17 +22,17 @@
 Perform the preview or delete operations
 """
 
+from __future__ import absolute_import, print_function
+
+from bleachbit import DeepScan, FileUtilities
+from bleachbit.Cleaner import backends
+from bleachbit import _, ungettext, expanduser
 
 import logging
 import math
-import os
 import sys
-import traceback
 
-import DeepScan
-import FileUtilities
-from Cleaner import backends
-from Common import _, ungettext
+logger = logging.getLogger(__name__)
 
 
 class Worker:
@@ -73,7 +73,6 @@ class Worker:
         # replaced by a message such as 'Permission denied.'
         err = _("Exception while running operation '%(operation)s': '%(msg)s'") \
             % {'operation': operation, 'msg': str(sys.exc_info()[1])}
-        logger = logging.getLogger(__name__)
         logger.error(err, exc_info=True)
         self.total_errors += 1
 
@@ -89,21 +88,19 @@ class Worker:
                     yield ret
         except SystemExit:
             pass
-        except Exception, e:
+        except Exception as e:
             # 2 = does not exist
             # 13 = permission denied
             from errno import ENOENT, EACCES
-            logger = logging.getLogger(__name__)
-            if (isinstance(e, OSError) and e.errno in (ENOENT, EACCES)):
+            if isinstance(e, OSError) and e.errno in (ENOENT, EACCES):
                 # For access denied, do not show traceback
-                logger.error('%s: %s' % (str(e), str(cmd)))
+                logger.error('%s: %s', e, cmd)
             else:
                 # For other errors, show the traceback.
-                logger.error('Error in execution of %s' %
-                             str(cmd), exc_info=True)
+                logger.error('Error in execution of %s', cmd, exc_info=True)
             self.total_errors += 1
         else:
-            if None == ret:
+            if ret is None:
                 return
             if isinstance(ret['size'], (int, long)):
                 size = FileUtilities.bytes_to_human(ret['size'])
@@ -129,9 +126,7 @@ class Worker:
         """Perform a single cleaning operation"""
         operation_options = self.operations[operation]
         assert(isinstance(operation_options, list))
-        logger = logging.getLogger(__name__)
-        logger.debug("clean_operation('%s'), options = '%s'" %
-                     (operation, operation_options))
+        logger.debug("clean_operation('%s'), options = '%s'", operation, operation_options)
 
         if not operation_options:
             raise StopIteration
@@ -170,11 +165,11 @@ class Worker:
             # deep scan
             for ds in backends[operation].get_deep_scan(option_id):
                 if '' == ds['path']:
-                    ds['path'] = os.path.expanduser('~')
+                    ds['path'] = expanduser('~')
                 if 'delete' != ds['command']:
                     raise NotImplementedError(
                         'Deep scan only supports deleting now')
-                if not self.deepscans.has_key(ds['path']):
+                if ds['path'] not in self.deepscans:
                     self.deepscans[ds['path']] = []
                 self.deepscans[ds['path']].append(ds)
         self.ui.update_item_size(operation, -1, total_size)
@@ -191,7 +186,6 @@ class Worker:
             raise RuntimeError("Unexpected option_id in delayed ops")
         self.ui.update_progress_bar(msg)
         for cmd in backends[operation].get_commands(option_id):
-            old_phase = None
             for ret in self.execute(cmd):
                 if isinstance(ret, tuple):
                     # Display progress (for free disk space)
@@ -242,9 +236,18 @@ class Worker:
                     self.delayed_ops.append(new_op)
 
         # standard operations
-        for dummy in self.run_operations(self.operations):
-            # yield to GTK+ idle loop
-            yield True
+        import warnings
+        with warnings.catch_warnings(record=True) as ws:
+            # This warning system allows general warnings. Duplicate will
+            # be removed, and the warnings will show near the end of
+            # the log.
+
+            warnings.simplefilter('once')
+            for dummy in self.run_operations(self.operations):
+                # yield to GTK+ idle loop
+                yield True
+            for w in ws:
+                logger.warning(w.message)
 
         # run deep scan
         if self.deepscans:
@@ -295,7 +298,6 @@ class Worker:
 
     def run_deep_scan(self):
         """Run deep scans"""
-        logger = logging.getLogger(__name__)
         logger.debug(' deepscans=%s' % self.deepscans)
         # TRANSLATORS: The "deep scan" feature searches over broad
         # areas of the file system such as the user's whole home directory
@@ -303,7 +305,7 @@ class Worker:
         self.ui.update_progress_bar(_("Please wait.  Running deep scan."))
         yield True  # allow GTK to update the screen
         ds = DeepScan.DeepScan()
-        for (path, dsdict) in self.deepscans.iteritems():
+        for (path, dsdict) in self.deepscans.items():
             logger.debug('deepscan path=%s, dict=%s' % (path, dsdict))
             for dsdict2 in dsdict:
                 ds.add_search(path, dsdict2['regex'])
@@ -313,7 +315,7 @@ class Worker:
                 yield True
                 continue
             # fixme: support non-delete commands
-            import Command
+            from bleachbit import Command
             cmd = Command.Delete(path)
             for ret in self.execute(cmd):
                 yield True
