@@ -2,8 +2,8 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2016 Andrew Ziem
-# http://www.bleachbit.org
+# Copyright (C) 2008-2018 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,29 +23,17 @@
 Test case for module Diagnostic
 """
 
-import sys
-import unittest
+from __future__ import absolute_import, print_function
 
-import common
-
-sys.path.append('.')
+from tests import common
 from bleachbit.Diagnostic import diagnostic_info
 
 
-class DiagnosticTestCase(unittest.TestCase):
-
+class DiagnosticTestCase(common.BleachbitTestCase):
     """Test Case for module Diagnostic"""
 
     def test_diagnostic_info(self):
         """Test diagnostic_info"""
         # at least it does not crash
         ret = diagnostic_info()
-        self.assert_(isinstance(ret, (str, unicode)))
-
-
-def suite():
-    return unittest.makeSuite(DiagnosticTestCase)
-
-
-if __name__ == '__main__':
-    unittest.main()
+        self.assertIsString(ret)
