@@ -2,8 +2,8 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2016 Andrew Ziem
-# http://www.bleachbit.org
+# Copyright (C) 2008-2018 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,56 +23,35 @@
 Test case for module FileUtilities
 """
 
-# for Python 2.5 on Windows
-from __future__ import with_statement
+from __future__ import absolute_import, print_function
 
-import locale
-import platform
-import subprocess
-import sys
-import tempfile
-import time
-import unittest
-
-import common
-
-sys.path.append('.')
+from tests import common
 from bleachbit.FileUtilities import *
 from bleachbit.Options import options
+from bleachbit import expanduser, expandvars, logger
 
-try:
-    import json
-except:
-    import simplejson as json
-
-
-def write_file(filename, contents):
-    """Write contents to file"""
-    with open(extended_path(filename), 'w') as f:
-        f.write(contents)
+import json
+import sys
+import unittest
 
 
 def test_ini_helper(self, execute):
     """Used to test .ini cleaning in TestAction and in TestFileUtilities"""
 
+    teststr = b'#Test\n[RecentsMRL]\nlist=C:\\Users\\me\\Videos\\movie.mpg,C:\\Users\\me\\movie2.mpg\n\n'
     # create test file
-    (fd, filename) = tempfile.mkstemp(prefix='bleachbit-test-ini')
-    os.write(fd, '#Test\n')
-    os.write(fd, '[RecentsMRL]\n')
-    os.write(
-        fd, 'list=C:\\Users\\me\\Videos\\movie.mpg,C:\\Users\\me\\movie2.mpg\n\n')
-    os.close(fd)
-    self.assert_(os.path.exists(filename))
+    filename = self.write_file('bleachbit-test-ini', teststr)
+    self.assertExists(filename)
     size = os.path.getsize(filename)
-    self.assertEqual(77, size)
+    self.assertEqual(len(teststr), size)
 
     # section does not exist
     execute(filename, 'Recents', None)
-    self.assertEqual(77, os.path.getsize(filename))
+    self.assertEqual(len(teststr), os.path.getsize(filename))
 
     # parameter does not exist
     execute(filename, 'RecentsMRL', 'files')
-    self.assertEqual(77, os.path.getsize(filename))
+    self.assertEqual(len(teststr), os.path.getsize(filename))
 
     # parameter does exist
     execute(filename, 'RecentsMRL', 'list')
@@ -84,33 +63,32 @@ def test_ini_helper(self, execute):
 
     # clean up
     delete(filename)
-    self.assert_(not os.path.exists(filename))
+    self.assertNotExists(filename)
 
 
 def test_json_helper(self, execute):
     """Used to test JSON cleaning in TestAction and in TestFileUtilities"""
 
     def load_js(js_fn):
-        js_fd = open(js_fn, 'r')
-        return json.load(js_fd)
+        with open(js_fn, 'r') as js_fd:
+            return json.load(js_fd)
+
+    expected = {'deleteme': 1, 'spareme': {'deletemetoo': 1}}
 
     # create test file
-    (fd, filename) = tempfile.mkstemp(prefix='bleachbit-test-json')
+    (fd, filename) = tempfile.mkstemp(prefix='bleachbit-test-json', dir=self.tempdir)
     os.write(fd, '{ "deleteme" : 1, "spareme" : { "deletemetoo" : 1 } }')
     os.close(fd)
-    self.assert_(os.path.exists(filename))
-    self.assertEqual(load_js(filename), {
-                     'deleteme': 1, 'spareme': {'deletemetoo': 1}})
+    self.assertExists(filename)
+    self.assertEqual(load_js(filename), expected)
 
     # invalid key
     execute(filename, 'doesnotexist')
-    self.assertEqual(load_js(filename), {
-                     'deleteme': 1, 'spareme': {'deletemetoo': 1}})
+    self.assertEqual(load_js(filename), expected)
 
     # invalid key
     execute(filename, 'deleteme/doesnotexist')
-    self.assertEqual(load_js(filename), {
-                     'deleteme': 1, 'spareme': {'deletemetoo': 1}})
+    self.assertEqual(load_js(filename), expected)
 
     # valid key
     execute(filename, 'deleteme')
@@ -126,11 +104,10 @@ def test_json_helper(self, execute):
 
     # clean up
     delete(filename)
-    self.assert_(not os.path.exists(filename))
+    self.assertNotExists(filename)
 
 
-class FileUtilitiesTestCase(unittest.TestCase):
-
+class FileUtilitiesTestCase(common.BleachbitTestCase):
     """Test case for module FileUtilities"""
 
     def test_bytes_to_human(self):
@@ -178,16 +155,15 @@ class FileUtilitiesTestCase(unittest.TestCase):
             human = bytes_to_human(bytes1)
             bytes2 = human_to_bytes(human)
             error = abs(float(bytes2 - bytes1) / bytes1)
-            self.assert_(abs(error) < 0.01,
-                         "%d (%s) is %.2f%% different than %d" %
-                         (bytes1, human, error * 100, bytes2))
+            self.assertLess(abs(error), 0.01, "%d (%s) is %.2f%% different than %d" %
+                            (bytes1, human, error * 100, bytes2))
 
         # test localization
         if hasattr(locale, 'format_string'):
             try:
                 locale.setlocale(locale.LC_NUMERIC, 'de_DE.utf8')
             except:
-                print "Warning: exception when setlocale to de_DE.utf8"
+                logger.warning('exception when setlocale to de_DE.utf8')
             else:
                 self.assertEqual("1,01GB", bytes_to_human(1000 ** 3 + 5812389))
 
@@ -199,19 +175,16 @@ class FileUtilitiesTestCase(unittest.TestCase):
         """Unit test for function children_in_directory()"""
 
         # test an existing directory that usually exists
-        dirname = os.path.expanduser("~/.config")
+        dirname = expanduser("~/.config")
         for filename in children_in_directory(dirname, True):
-            self.assert_(isinstance(filename, str))
-            self.assert_(os.path.isabs(filename))
+            self.assertTrue(os.path.isabs(filename))
         for filename in children_in_directory(dirname, False):
-            self.assert_(isinstance(filename, str))
-            self.assert_(os.path.isabs(filename))
-            self.assert_(not os.path.isdir(filename))
+            self.assertTrue(os.path.isabs(filename))
+            self.assertFalse(os.path.isdir(filename))
 
         # test a constructed file in a constructed directory
-        dirname = tempfile.mkdtemp(prefix='bleachbit-test-children')
-        filename = os.path.join(dirname, "somefile")
-        common.touch_file(filename)
+        dirname = self.mkdtemp(prefix='bleachbit-test-children')
+        filename = self.mkstemp(prefix="somefile", dir=dirname)
         for loopfilename in children_in_directory(dirname, True):
             self.assertEqual(loopfilename, filename)
         for loopfilename in children_in_directory(dirname, False):
@@ -224,36 +197,36 @@ class FileUtilitiesTestCase(unittest.TestCase):
         for filename in children_in_directory(dirname, True):
             self.assertEqual(filename, subdirname)
         for filename in children_in_directory(dirname, False):
-            self.assert_(False)
+            raise AssertionError('Found a file that shouldn\'t have been found: ' + filename)
         os.rmdir(subdirname)
 
         os.rmdir(dirname)
 
     def test_clean_ini(self):
         """Unit test for clean_ini()"""
-        print "testing test_clean_ini() with shred = False"
+        print("testing test_clean_ini() with shred = False")
         options.set('shred', False, commit=False)
         test_ini_helper(self, clean_ini)
 
-        print "testing test_clean_ini() with shred = True"
+        print("testing test_clean_ini() with shred = True")
         options.set('shred', True, commit=False)
         test_ini_helper(self, clean_ini)
 
     def test_clean_json(self):
         """Unit test for clean_json()"""
-        print "testing test_clean_json() with shred = False"
+        print("testing test_clean_json() with shred = False")
         options.set('shred', False, commit=False)
         test_json_helper(self, clean_json)
 
-        print "testing test_clean_json() with shred = True"
+        print("testing test_clean_json() with shred = True")
         options.set('shred', True, commit=False)
         test_json_helper(self, clean_json)
 
     def test_delete(self):
         """Unit test for method delete()"""
-        print "testing delete() with shred = False"
+        print("testing delete() with shred = False")
         self.delete_helper(shred=False)
-        print "testing delete() with shred = True"
+        print("testing delete() with shred = True")
         self.delete_helper(shred=True)
         # exercise ignore_missing
         delete('does-not-exist', ignore_missing=True)
@@ -267,108 +240,97 @@ class FileUtilitiesTestCase(unittest.TestCase):
         katanana = u"アメリカ"
         umlauts = u"ÄäǞǟËëḦḧÏïḮḯÖöȪȫṎṏT̈ẗÜüǕǖǗǘǙǚǛǜṲṳṺṻẄẅẌẍŸÿ"
 
-        tests = [('.prefix', 'suffix'),  # simple
-                 ("x".zfill(100), ".y".zfill(50)),  # long
-                 (' ', 'begins_with_space'),
-                 ("'", "'"),  # quotation mark
-                 ("~`!@#$%^&()-_+=", "x"),  # non-alphanumeric characters
-                 ("[]{};'.,", "x"),  # non-alphanumeric characters
-                 (u'abcd', u'efgh'),  # simple Unicode
-                 (u'J\xf8rgen', 'Scandinavian'),
-                 (u'\u2014', 'em-dash'),  # LP#1454030
-                 (hebrew, hebrew),
-                 (katanana, katanana),
-                 (umlauts, umlauts),
-                 ('sigil', 'should$not-change')]
+        tests = ['.prefixandsuffix',  # simple
+                 "x".zfill(150),  # long
+                 ' begins_with_space',
+                 "''",  # quotation mark
+                 "~`!@#$%^&()-_+=x",  # non-alphanumeric characters
+                 "[]{};'.,x",  # non-alphanumeric characters
+                 u'abcdefgh',  # simple Unicode
+                 u'J\xf8rgen Scandinavian',
+                 u'\u2014em-dash',  # LP#1454030
+                 hebrew,
+                 katanana,
+                 umlauts,
+                 'sigil-should$not-change']
         if 'posix' == os.name:
             # Windows doesn't allow these characters but Unix systems do
-            tests.append(('"', '*'))
-            tests.append(('\t', '\\'))
-            tests.append((':?', '<>|'))
-            # Windows filenames cannot end with space or period
-            tests.append((' ', ' '))
-            tests.append(('.', '.'))
+            tests += ['"*', '\t\\', ':?<>|',
+                      ' ', '.file.'] # Windows filenames cannot end with space or period
         for test in tests:
-            # delete a file
-            (fd, filename) = tempfile.mkstemp(
-                prefix='bleachbit-test-delete-file' + test[0], suffix=test[1])
-            self.assert_(os.path.exists(filename))
-            for x in range(0, 4096):
-                bytes_written = os.write(fd, "top secret")
-                self.assertEqual(bytes_written, 10)
-            os.close(fd)
-            self.assert_(os.path.exists(filename))
+            # create the file
+            filename = self.write_file(test, "top secret")
+            # delete the file
             delete(filename, shred)
-            self.assert_(not os.path.exists(filename))
+            self.assertNotExists(filename)
 
             # delete an empty directory
-            dirname = tempfile.mkdtemp(
-                prefix='bleachbit-test-delete-dir' + test[0], suffix=test[1])
-            self.assert_(os.path.exists(dirname))
+            dirname = self.mkdtemp(prefix=test)
+            self.assertExists(dirname)
             delete(dirname, shred)
-            self.assert_(not os.path.exists(dirname))
+            self.assertNotExists(dirname)
 
         def symlink_helper(link_fn):
-
             if 'nt' == os.name:
                 from win32com.shell import shell
                 if not shell.IsUserAnAdmin():
-                    print 'WARNING: skipping symlink test because of insufficient privileges'
-                    return
+                    self.skipTest('skipping symlink test because of insufficient privileges')
 
             # make regular file
-            (fd, srcname) = tempfile.mkstemp(
-                prefix='bleachbit-test-delete-regular')
-            os.close(fd)
+            srcname = self.mkstemp(prefix='bleachbit-test-delete-regular')
 
             # make symlink
-            self.assert_(os.path.exists(srcname))
+            self.assertExists(srcname)
             linkname = tempfile.mktemp('bblink')
-            self.assert_(not os.path.exists(linkname))
+            self.assertNotExists(linkname)
             link_fn(srcname, linkname)
-            self.assert_(os.path.exists(linkname))
-            self.assert_(os.path.lexists(linkname))
+            self.assertExists(linkname)
+            self.assertLExists(linkname)
 
             # delete symlink
             delete(linkname, shred)
-            self.assert_(os.path.exists(srcname))
-            self.assert_(not os.path.lexists(linkname))
+            self.assertExists(srcname)
+            self.assertNotLExists(linkname)
 
             # delete regular file
             delete(srcname, shred)
-            self.assert_(not os.path.exists(srcname))
+            self.assertNotExists(srcname)
 
             #
             # test broken symlink
             #
-            (fd, srcname) = tempfile.mkstemp(
-                prefix='bleachbit-test-delete-sym')
-            os.close(fd)
-            self.assert_(os.path.lexists(srcname))
+            srcname = self.mkstemp(prefix='bleachbit-test-delete-sym')
+            self.assertLExists(srcname)
             link_fn(srcname, linkname)
-            self.assert_(os.path.lexists(linkname))
-            self.assert_(os.path.exists(linkname))
+            self.assertLExists(linkname)
+            self.assertExists(linkname)
 
             # delete regular file first
             delete(srcname, shred)
-            self.assert_(not os.path.exists(srcname))
+            self.assertNotExists(srcname)
+            self.assertLExists(linkname)
 
             # clean up
             delete(linkname, shred)
-            self.assert_(not os.path.exists(linkname))
-            self.assert_(not os.path.lexists(linkname))
+            self.assertNotExists(linkname)
+            self.assertNotLExists(linkname)
 
-        if 'nt' == os.name and platform.uname()[3][0:3] > '5.1':
-            # Windows XP = 5.1
-            # test symlinks
+        windows_vista_or_newer = False
+        if 'nt' == os.name:
+            from bleachbit.Windows import parse_windows_build
+            # Windows Vista = 6.0
+            windows_vista_or_newer = parse_windows_build() >= 6.0
+
+        if windows_vista_or_newer:
+            logger.debug('testing symbolic link')
             import ctypes
             kern = ctypes.windll.LoadLibrary("kernel32.dll")
 
             def win_symlink(src, linkname):
                 rc = kern.CreateSymbolicLinkA(linkname, src, 0)
                 if rc == 0:
-                    print 'CreateSymbolicLinkA(%s, %s)' % (linkname, src)
-                    print 'CreateSymolicLinkA() failed, error = %s' % ctypes.FormatError()
+                    print('CreateSymbolicLinkA(%s, %s)' % (linkname, src))
+                    print('CreateSymolicLinkA() failed, error = %s' % ctypes.FormatError())
                     self.assertNotEqual(rc, 0)
             symlink_helper(win_symlink)
 
@@ -377,33 +339,73 @@ class FileUtilitiesTestCase(unittest.TestCase):
             return
 
         # test file with mode 0444/-r--r--r--
-        (fd, filename) = tempfile.mkstemp(prefix='bleachbit-test-0444')
-        os.close(fd)
-        os.chmod(filename, 0444)
+        filename = self.write_file('bleachbit-test-0444')
+        os.chmod(filename, 0o444)
         delete(filename, shred)
-        self.assert_(not os.path.exists(filename))
+        self.assertNotExists(filename)
 
         # test symlink
         symlink_helper(os.symlink)
 
-        # test fifo
+        # test FIFO
         args = ["mkfifo", filename]
         ret = subprocess.call(args)
         self.assertEqual(ret, 0)
-        self.assert_(os.path.exists(filename))
+        self.assertExists(filename)
         delete(filename, shred)
-        self.assert_(not os.path.exists(filename))
+        self.assertNotExists(filename)
 
         # test directory
-        path = tempfile.mkdtemp(prefix='bleachbit-test-delete-dir')
-        self.assert_(os.path.exists(path))
+        path = self.mkdtemp(prefix='bleachbit-test-delete-dir')
+        self.assertExists(path)
         delete(path, shred)
-        self.assert_(not os.path.exists(path))
+        self.assertNotExists(path)
 
+    @unittest.skipIf('nt' != os.name, 'skipping on non-Windows')
+    def test_delete_locked(self):
+        """Unit test for delete() with locked file"""
+        # set up
+        def test_delete_locked_setup():
+            (fd, filename) = tempfile.mkstemp(prefix='bleachbit-test-worker')
+            os.write(fd, '123')
+            os.close(fd)
+            self.assertExists(filename)
+            self.assertEqual(3, getsize(filename))
+            return filename
+
+        # File is open but not opened exclusive, so expect that the
+        # file is truncated but not deleted.
+        # O_EXCL = fail if file exists (i.e., not an exclusive lock)
+        filename = test_delete_locked_setup()
+        f = os.open(filename, os.O_WRONLY | os.O_EXCL)
+        self.assertExists(filename)
+        self.assertEqual(3, getsize(filename))
+        with self.assertRaises(WindowsError):
+            delete(filename)
+        os.close(f)
+        self.assertExists(filename)
+        self.assertEqual(0, getsize(filename))
+        delete(filename)
+        self.assertNotExists(filename)
+
+        # File is open with exclusive lock, so expect the file is neither
+        # deleted nor truncated.
+        filename = test_delete_locked_setup()
+        from subprocess import call
+        call('start notepad.exe>>{}'.format(filename), shell=True)
+        self.assertExists(filename)
+        self.assertEqual(3, getsize(filename))
+        with self.assertRaises(WindowsError):
+            delete(filename)
+        call('taskkill /f /im notepad.exe')
+        self.assertExists(filename)
+        self.assertEqual(3, getsize(filename))
+        delete(filename)
+        self.assertNotExists(filename)
+
+    @unittest.skipIf('nt' == os.name, 'skipping on Windows')
     def test_ego_owner(self):
         """Unit test for ego_owner()"""
-        if 'nt' == os.name:
-            return
         self.assertEqual(ego_owner('/bin/ls'), os.getuid() == 0)
 
     def test_exists_in_path(self):
@@ -411,7 +413,7 @@ class FileUtilitiesTestCase(unittest.TestCase):
         filename = 'ls'
         if 'nt' == os.name:
             filename = 'cmd.exe'
-        self.assert_(exists_in_path(filename))
+        self.assertTrue(exists_in_path(filename))
 
     def test_exe_exists(self):
         """Unit test for exe_exists()"""
@@ -434,13 +436,37 @@ class FileUtilitiesTestCase(unittest.TestCase):
         if 'nt' == os.name:
             expand_glob_join('c:\windows', '*.exe')
 
+    def test_expandvars(self):
+        """Unit test for expandvars()."""
+        expanded = expandvars('$HOME')
+        self.assertIsUnicodeString(expanded)
+
+    def test_extended_path(self):
+        """Unit test for extended_path() and extended_path_undo()"""
+        if 'nt' == os.name:
+            tests = [
+                (r'c:\windows\notepad.exe', r'\\?\c:\windows\notepad.exe'),
+                (r'\\server\share\windows\notepad.exe', r'\\?\unc\server\share\windows\notepad.exe'),
+            ]
+        else:
+            # unchanged
+            tests = (('/home/foo', '/home/foo'),)
+        for short, extended in tests:
+            # already extended path shouldn't be changed
+            self.assertEqual(extended_path(extended), extended)
+            # does the conversion work both ways?
+            self.assertEqual(extended_path(short), extended)
+            self.assertEqual(extended_path_undo(extended), short)
+            # unextended paths shouldn't be shortened any more
+            self.assertEqual(extended_path_undo(short), short)
+
     def test_free_space(self):
         """Unit test for free_space()"""
-        home = os.path.expanduser("~")
+        home = expanduser('~')
         result = free_space(home)
         self.assertNotEqual(result, None)
-        self.assert_(result > -1)
-        self.assert_(isinstance(result, (int, long)))
+        self.assertGreater(result, -1)
+        self.assertIsInteger(result)
 
         # compare to WMIC
         if 'nt' != os.name:
@@ -449,7 +475,7 @@ class FileUtilitiesTestCase(unittest.TestCase):
         from bleachbit.General import run_external
         (rc, stdout, stderr) = run_external(args)
         if rc:
-            print 'error calling WMIC\nargs=%s\nstderr=%s' % (args, stderr)
+            print('error calling WMIC\nargs=%s\nstderr=%s' % (args, stderr))
             return
         import re
         for line in stdout.split('\n'):
@@ -457,16 +483,17 @@ class FileUtilitiesTestCase(unittest.TestCase):
             if not re.match('([A-Z]):\s+(\d+)', line):
                 continue
             drive, bytes_free = re.split('\s+', line)
+            print('Checking free space for %s' % drive)
             bytes_free = int(bytes_free)
-            self.assertEqual(bytes_free, free_space(drive))
+            free = free_space(unicode(drive))
+            self.assertEqual(bytes_free, free)
 
     def test_getsize(self):
         """Unit test for method getsize()"""
-        dirname = tempfile.mkdtemp(prefix='bleachbit-test-getsize')
+        dirname = self.mkdtemp(prefix='bleachbit-test-getsize')
 
         def test_getsize_helper(fname):
-            filename = os.path.join(dirname, fname)
-            write_file(filename, "abcdefghij" * 12345)
+            filename = self.write_file(os.path.join(dirname, fname), "abcdefghij" * 12345)
 
             if 'nt' == os.name:
                 self.assertEqual(getsize(filename), 10 * 12345)
@@ -487,12 +514,12 @@ class FileUtilitiesTestCase(unittest.TestCase):
                     ["du", "-h", filename], stdout=subprocess.PIPE).communicate()[0]
                 output = output.replace("\n", "")
                 du_size = output.split("\t")[0] + "B"
-                print "output = '%s', size='%s'" % (output, du_size)
+                print("output = '%s', size='%s'" % (output, du_size))
                 du_bytes = human_to_bytes(du_size, 'du')
-                print output, du_size, du_bytes
+                print(output, du_size, du_bytes)
                 self.assertEqual(getsize(filename), du_bytes)
             delete(filename)
-            self.assert_(not os.path.exists(filename))
+            self.assertNotExists(filename)
 
         # create regular file
         test_getsize_helper('bleachbit-test-regular')
@@ -514,17 +541,18 @@ class FileUtilitiesTestCase(unittest.TestCase):
             return
 
         # create a symlink
-        (handle, filename) = tempfile.mkstemp(prefix='bleachbit-test-symlink')
-        os.write(handle, "abcdefghij" * 12345)
-        os.close(handle)
-        linkname = '/tmp/bleachbitsymlinktest'
+        filename = self.write_file('bleachbit-test-symlink', 'abcdefghij' * 12345)
+        linkname = os.path.join(self.tempdir, 'bleachbitsymlinktest')
         if os.path.lexists(linkname):
             delete(linkname)
         os.symlink(filename, linkname)
-        self.assert_(getsize(linkname) < 8192, "Symlink size is %d" %
-                     getsize(filename))
+        self.assertLess(getsize(linkname), 8192, "Symlink size is %d" % getsize(filename))
         delete(filename)
 
+        if 'darwin' == sys.platform:
+            # MacOS's HFS+ filesystem doesn't support sparse files
+            return
+
         # create sparse file
         (handle, filename) = tempfile.mkstemp(prefix="bleachbit-test-sparse")
         os.ftruncate(handle, 1000 ** 2)
@@ -537,7 +565,7 @@ class FileUtilitiesTestCase(unittest.TestCase):
         path = '/bin'
         if 'nt' == os.name:
             path = 'c:\\windows\\system32'
-        self.assert_(getsizedir(path) > 0)
+        self.assertGreater(getsizedir(path), 0)
 
     def test_globex(self):
         """Unit test for method globex()"""
@@ -547,27 +575,64 @@ class FileUtilitiesTestCase(unittest.TestCase):
     def test_guess_overwrite_paths(self):
         """Unit test for guess_overwrite_paths()"""
         for path in guess_overwrite_paths():
-            self.assert_(os.path.isdir(path), '%s is not a directory' % path)
+            self.assertTrue(os.path.isdir(path), '%s is not a directory' % path)
+
+    def test_human_to_bytes(self):
+        """Unit test for human_to_bytes()"""
+        self.assertRaises(ValueError, human_to_bytes, '', hformat='invalid')
+
+        for test in ['Bazillion kB', '120XB', '.12MB']:
+            self.assertRaises(ValueError, human_to_bytes, test)
+
+        valid = {'1kB': 1000,
+                 '1.1MB': 1100000,
+                 '12B': 12,
+                 '1.0M': 1000*1000,
+                 '1TB': 1000**4,
+                 '1000': 1000}
+        for test, result in valid.items():
+            self.assertEqual(human_to_bytes(test), result)
+
+        self.assertEqual(human_to_bytes('1 MB', 'du'), 1024*1024)
 
     def test_listdir(self):
         """Unit test for listdir()"""
-        for pathname in listdir(('/tmp', '~/.config/')):
-            self.assert_(os.path.lexists(pathname),
-                         "does not exist: %s" % pathname)
+        if 'posix' == os.name:
+            dir1 = '/tmp'
+            dir2 = expanduser('~/.config')
+        if 'nt' == os.name:
+            dir1 = expandvars(r'%windir%\fonts')
+            dir2 = expandvars(r'%userprofile%\desktop')
+        # If these directories do not exist, the test results are not valid.
+        self.assertExists(dir1)
+        self.assertExists(dir2)
+        # Every path found in dir1 and dir2 should be found in (dir1, dir2).
+        paths1 = set(listdir(dir1))
+        paths2 = set(listdir(dir2))
+        paths12 = set(listdir((dir1,dir2)))
+        self.assertTrue(paths1 < paths12)
+        self.assertTrue(paths2 < paths12)
+        # The individual calls should be equivalent to a combined call.
+        self.assertSetEqual(paths1.union(paths2), paths12)
+        # The directories should not be empty.
+        self.assertGreater(len(paths1), 0)
+        self.assertGreater(len(paths2), 0)
+        # Every path found should exist.
+        for pathname in paths12:
+            self.assertLExists(pathname)
 
     def test_same_partition(self):
         """Unit test for same_partition()"""
-        home = os.path.expanduser('~')
+        home = expanduser('~')
         self.assertTrue(same_partition(home, home))
         if 'posix' == os.name:
-            self.assertFalse(same_partition(home, '/proc'))
+            self.assertFalse(same_partition(home, '/dev'))
         if 'nt' == os.name:
             home_drive = os.path.splitdrive(home)[0]
             from bleachbit.Windows import get_fixed_drives
             for drive in get_fixed_drives():
                 this_drive = os.path.splitdrive(drive)[0]
-                self.assertEqual(
-                    same_partition(home, drive), home_drive == this_drive)
+                self.assertEqual(same_partition(home, drive), home_drive == this_drive)
 
     def test_whitelisted(self):
         """Unit test for whitelisted()"""
@@ -604,7 +669,7 @@ class FileUtilitiesTestCase(unittest.TestCase):
             self.assertTrue(whitelisted('C:\\WINDOWS\\FOO.LOG'))
             self.assertTrue(whitelisted('D:\\USERS'))
 
-            # drives letters have the seperator at the end while most paths
+            # drives letters have the separator at the end while most paths
             # don't
             self.assertTrue(whitelisted('D:\\FOLDER\\FOO.LOG'))
 
@@ -624,13 +689,66 @@ class FileUtilitiesTestCase(unittest.TestCase):
         self.assertEqual(
             set(old_whitelist), set(options.get_whitelist_paths()))
 
+    @unittest.skipUnless('posix' == os.name, 'skipping on non-POSIX platform')
+    def test_whitelisted_posix_symlink(self):
+        """Symlink test for whitelisted_posix()"""
+        # setup
+        old_whitelist = options.get_whitelist_paths()
+        tmpdir = os.path.join(self.tempdir, 'bleachbit-whitelist')
+        os.mkdir(tmpdir)
+        realpath = self.write_file('real')
+        linkpath = os.path.join(tmpdir, 'link')
+        os.symlink(realpath, linkpath)
+        self.assertExists(realpath)
+        self.assertExists(linkpath)
+
+        # test 1: the real path is whitelisted
+        whitelist = [('file', realpath)]
+        options.set_whitelist_paths(whitelist)
+        self.assertFalse(whitelisted(tmpdir))
+        self.assertTrue(whitelisted(realpath))
+        self.assertTrue(whitelisted(linkpath))
+
+        # test 2: the link is whitelisted
+        whitelist = [('file', linkpath)]
+        options.set_whitelist_paths(whitelist)
+        self.assertFalse(whitelisted(tmpdir))
+        self.assertFalse(whitelisted(realpath))
+        self.assertTrue(whitelisted(linkpath))
+
+        options.set_whitelist_paths(old_whitelist)
+
+    def test_whitelisted_speed(self):
+        """Benchmark the speed of whitelisted()
+
+        It is called frequently, so the speed is important."""
+        d = '/usr/bin'
+        whitelist = [('file', '/home/foo'), ('folder', '/home/folder')]
+        if 'nt' == os.name:
+            d = expandvars('%windir%\system32')
+            whitelist = [('file', r'c:\\filename'), ('folder', r'c:\\folder')]
+        reps = 20
+        paths = [p for p in children_in_directory(d, True)]
+        paths = paths[:1000]  # truncate
+        self.assertGreater(len(paths), 10)
+        old_whitelist = options.get_whitelist_paths()
+        options.set_whitelist_paths(whitelist)
+
+        t0 = time.time()
+        for i in xrange(0, reps):
+            for p in paths:
+                _ = whitelisted(p)
+        t1 = time.time()
+        logger.info('whitelisted() with {} repetitions and {} paths took {:.3g} seconds '.format(
+            reps, len(paths), t1 - t0))
+
+        options.set_whitelist_paths(old_whitelist)
+
     def test_wipe_contents(self):
         """Unit test for wipe_delete()"""
 
         # create test file
-        (handle, filename) = tempfile.mkstemp(prefix="bleachbit-test-wipe")
-        os.write(handle, "abcdefghij" * 12345)
-        os.close(handle)
+        filename = self.write_file('bleachbit-test-wipe', 'abcdefghij' * 12345)
 
         # wipe it
         wipe_contents(filename)
@@ -650,84 +768,70 @@ class FileUtilitiesTestCase(unittest.TestCase):
     def wipe_name_helper(self, filename):
         """Helper for test_wipe_name()"""
 
-        self.assert_(os.path.exists(filename))
+        self.assertExists(filename)
 
         # test
         newname = wipe_name(filename)
-        self.assert_(len(filename) > len(newname))
-        self.assert_(not os.path.exists(filename))
-        self.assert_(os.path.exists(newname))
+        self.assertGreater(len(filename), len(newname))
+        self.assertNotExists(filename)
+        self.assertExists(newname)
 
         # clean
         os.remove(newname)
-        self.assert_(not os.path.exists(newname))
+        self.assertNotExists(newname)
 
     def test_wipe_name(self):
         """Unit test for wipe_name()"""
 
         # create test file with moderately long name
-        (handle, filename) = tempfile.mkstemp(
-            prefix="bleachbit-test-wipe" + "0" * 50)
-        os.close(handle)
+        filename = self.write_file('bleachbit-test-wipe' + '0' * 50)
         self.wipe_name_helper(filename)
 
         # create file with short name in temporary directory with long name
-        if 'posix' == os.name:
-            dir0len = 210
-            dir1len = 210
-            filelen = 10
         if 'nt' == os.name:
             # In Windows, the maximum path length is 260 characters
             # http://msdn.microsoft.com/en-us/library/aa365247%28VS.85%29.aspx#maxpath
             dir0len = 100
             dir1len = 5
-            filelen = 10
+        else:
+            dir0len = 210
+            dir1len = 210
+        filelen = 10
 
-        dir0 = tempfile.mkdtemp(prefix="0" * dir0len)
-        self.assert_(os.path.exists(dir0))
+        dir0 = self.mkdtemp(prefix="0" * dir0len)
+        self.assertExists(dir0)
 
-        dir1 = tempfile.mkdtemp(prefix="1" * dir1len, dir=dir0)
-        self.assert_(os.path.exists(dir1))
+        dir1 = self.mkdtemp(prefix="1" * dir1len, dir=dir0)
+        self.assertExists(dir1)
 
-        (handle, filename) = tempfile.mkstemp(
-            dir=dir1, prefix="2" * filelen)
-        os.close(handle)
+        filename = self.write_file(os.path.join(dir1, '2' * filelen))
         self.wipe_name_helper(filename)
-        self.assert_(os.path.exists(dir0))
-        self.assert_(os.path.exists(dir1))
+        self.assertExists(dir0)
+        self.assertExists(dir1)
 
         # wipe a directory name
         dir1new = wipe_name(dir1)
-        self.assert_(len(dir1) > len(dir1new))
-        self.assert_(not os.path.exists(dir1))
-        self.assert_(os.path.exists(dir1new))
+        self.assertGreater(len(dir1), len(dir1new))
+        self.assertNotExists(dir1)
+        self.assertExists(dir1new)
         os.rmdir(dir1new)
 
         # wipe the directory
         os.rmdir(dir0)
-        self.assert_(not os.path.exists(dir0))
+        self.assertNotExists(dir0)
 
+    @unittest.skipUnless(os.getenv('ALLTESTS') is not None,
+                         'warning: skipping long test test_wipe_path() because environment variable ALLTESTS not set')
     def test_wipe_path(self):
         """Unit test for wipe_path()"""
-        if None == os.getenv('ALLTESTS'):
-            print 'warning: skipping long test test_wipe_path() because environment variable ALLTESTS not set'
-            return
-        pathname = tempfile.gettempdir()
-        for ret in wipe_path(pathname):
+
+        for ret in wipe_path(self.tempdir):
             # no idle handler
             pass
 
     def test_vacuum_sqlite3(self):
         """Unit test for method vacuum_sqlite3()"""
-
-        try:
-            import sqlite3
-        except ImportError, e:
-            if sys.version_info[0] == 2 and sys.version_info[1] < 5:
-                print "Warning: Skipping test_vacuum_sqlite3() on old Python"
-                return
-            else:
-                raise e
+        import sqlite3
 
         path = os.path.abspath('bleachbit.tmp.sqlite3')
         if os.path.lexists(path):
@@ -741,10 +845,9 @@ class FileUtilitiesTestCase(unittest.TestCase):
         def number_generator():
             for x in range(1, 10000):
                 yield (x, )
-        conn.executemany(
-            'insert into numbers (number) values ( ? ) ', number_generator())
+        conn.executemany('insert into numbers (number) values ( ? ) ', number_generator())
         conn.commit()
-        self.assert_(empty_size < getsize(path))
+        self.assertLess(empty_size, getsize(path))
         conn.execute('delete from numbers')
         conn.commit()
         conn.close()
@@ -754,13 +857,12 @@ class FileUtilitiesTestCase(unittest.TestCase):
 
         delete(path)
 
+    @unittest.skipIf('nt' == os.name, 'skipping on Windows')
     def test_OpenFiles(self):
         """Unit test for class OpenFiles"""
-        if 'nt' == os.name:
-            return
 
-        (handle, filename) = tempfile.mkstemp(
-            prefix='bleachbit-test-open-files')
+        filename = os.path.join(self.tempdir, 'bleachbit-test-open-files')
+        f = open(filename, 'w')
         openfiles = OpenFiles()
         self.assertTrue(openfiles.is_open(filename),
                         "Expected is_open(%s) to return True)\n"
@@ -770,7 +872,6 @@ class FileUtilitiesTestCase(unittest.TestCase):
                          time.time() - openfiles.last_scan_time,
                          openfiles.files))
 
-        f = os.fdopen(handle)
         f.close()
         openfiles.scan()
         self.assertFalse(openfiles.is_open(filename))
@@ -779,10 +880,5 @@ class FileUtilitiesTestCase(unittest.TestCase):
         openfiles.scan()
         self.assertFalse(openfiles.is_open(filename))
 
-
-def suite():
-    return unittest.makeSuite(FileUtilitiesTestCase)
-
-
-if __name__ == '__main__':
-    unittest.main()
+    def test_open_files_lsof(self):
+        self.assertEqual(list(open_files_lsof(lambda: 'n/bar/foo\nn/foo/bar\nnoise')), ['/bar/foo', '/foo/bar'])
