@@ -1,8 +1,8 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2016 Andrew Ziem
-# http://www.bleachbit.org
+# Copyright (C) 2008-2018 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,16 +22,13 @@
 Test case for RecognizeCleanerML
 """
 
+from __future__ import absolute_import, print_function
 
-import sys
-import unittest
-
-sys.path.append('.')
+from tests import common
 from bleachbit.RecognizeCleanerML import hashdigest
 
 
-class RecognizeCleanerMLTestCase(unittest.TestCase):
-
+class RecognizeCleanerMLTestCase(common.BleachbitTestCase):
     """Test case for RecognizeCleanerML"""
 
     def test_hash(self):
@@ -40,10 +37,3 @@ class RecognizeCleanerMLTestCase(unittest.TestCase):
         self.assertEqual(len(digest), 128)
         self.assertEqual(digest[1:10], '6382c203e')
 
-
-def suite():
-    return unittest.makeSuite(RecognizeCleanerMLTestCase)
-
-
-if __name__ == '__main__':
-    unittest.main()
