@@ -1,8 +1,8 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2016 Andrew Ziem
-# http://www.bleachbit.org
+# Copyright (C) 2008-2018 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,11 +17,16 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-
 """
 Test cases for module Winapp
 """
 
+from __future__ import absolute_import, print_function
+
+from tests import common
+from bleachbit.Winapp import Winapp, detectos, detect_file, section2option
+from bleachbit.Windows import detect_registry_key, parse_windows_build
+from bleachbit import logger
 
 import os
 import shutil
@@ -29,12 +34,6 @@ import sys
 import tempfile
 import unittest
 
-sys.path.append('.')
-from bleachbit.Winapp import Winapp, detectos, detect_file, section2option
-from bleachbit.Windows import detect_registry_key, parse_windows_build
-
-import common
-
 
 def CreateSubKey(sub_key):
     import _winreg
@@ -47,7 +46,7 @@ keyfull = 'HKCU\\Software\\BleachBit\\DeleteThisKey'
 
 def get_winapp2():
     """Download and cache winapp2.ini.  Return local filename."""
-    url = "https://rawgit.com/bleachbit/winapp2.ini/master/Winapp2-combined.ini"
+    url = "https://rawgit.com/bleachbit/winapp2.ini/master/Winapp2-BleachBit.ini"
     tmpdir = None
     if 'posix' == os.name:
         tmpdir = '/tmp'
@@ -59,18 +58,18 @@ def get_winapp2():
         import stat
         age_seconds = time.time() - os.stat(fn)[stat.ST_MTIME]
         if age_seconds > (24 * 36 * 36):
-            print 'note: deleting stale file %s ' % fn
+            logger.info('deleting stale file %s ', fn)
             os.remove(fn)
     if not os.path.exists(fn):
-        f = file(fn, 'w')
+        f = open(fn, 'w')
         import urllib2
         txt = urllib2.urlopen(url).read()
         f.write(txt)
     return fn
 
 
-class WinappTestCase(unittest.TestCase, common.AssertFile):
-
+@unittest.skipUnless('win32' == sys.platform, 'not running on windows')
+class WinappTestCase(common.BleachbitTestCase):
     """Test cases for Winapp"""
 
     def run_all(self, cleaner, really_delete):
@@ -146,7 +145,7 @@ class WinappTestCase(unittest.TestCase, common.AssertFile):
             for pathname in dir_32_unique:
                 tests.append(('%%ProgramFiles%%\\%s' % pathname, True))
         else:
-            print 'NOTE: skipping %ProgramW6432% tests because WoW64 not detected'
+            logger.info('skipping %ProgramW6432% tests because WoW64 not detected')
         for (pathname, expected_return) in tests:
             actual_return = detect_file(pathname)
             msg = 'detect_file(%s) returned %s' % (pathname, actual_return)
@@ -158,15 +157,15 @@ class WinappTestCase(unittest.TestCase, common.AssertFile):
 
         dirname = tempfile.mkdtemp(prefix='bleachbit-test-winapp')
         f1 = os.path.join(dirname, f1_filename or 'deleteme.log')
-        file(f1, 'w').write('')
+        open(f1, 'w').close()
 
         dirname2 = os.path.join(dirname, 'sub')
         os.mkdir(dirname2)
         f2 = os.path.join(dirname2, 'deleteme.log')
-        file(f2, 'w').write('')
+        open(f2, 'w').close()
 
         fbak = os.path.join(dirname, 'deleteme.bak')
-        file(fbak, 'w').write('')
+        open(fbak, 'w').close()
 
         self.assertExists(f1)
         self.assertExists(f2)
@@ -177,11 +176,11 @@ class WinappTestCase(unittest.TestCase, common.AssertFile):
         self.assertTrue(detect_registry_key(keyfull))
         self.assertTrue(detect_registry_key('HKCU\\%s' % subkey))
 
-        return (dirname, f1, f2, fbak)
+        return dirname, f1, f2, fbak
 
     def ini2cleaner(self, body, do_next=True):
         """Write a minimal Winapp2.ini"""
-        ini = file(self.ini_fn, 'w')
+        ini = open(self.ini_fn, 'w')
         ini.write('[someapp]\n')
         ini.write('LangSecRef=3021\n')
         ini.write(body)
@@ -248,6 +247,8 @@ class WinappTestCase(unittest.TestCase, common.AssertFile):
                 "\nDetectFile1=%%APPDATA%%\\Microsoft\nDetectFile2=%%APPDATA%%\\does_not_exist",
                 "\nDetectFile1=%%APPDATA%%\\does_not_exist\nDetectFile2=%%APPDATA%%\\Microsoft",
                 "\nDetect=HKCU\\Software\\Microsoft",
+                # Below checks that a space is OK in the registry key
+                "\nDetect=HKCU\\Software\\Microsoft\\Command Processor",
                 "\nDetect1=HKCU\\Software\\Microsoft\nDetect2=HKCU\\Software\\does_not_exist",
                     "\nDetect1=HKCU\\Software\\does_not_exist\nDetect2=HKCU\\Software\\Microsoft"):
                 new_ini = test[0] + detect
@@ -257,7 +258,7 @@ class WinappTestCase(unittest.TestCase, common.AssertFile):
 
         # execute positive tests
         for test in positive_tests:
-            print 'positive test: ', test
+            print('positive test: ', test)
             (dirname, f1, f2, fbak) = self.setup_fake(test[1])
             cleaner = self.ini2cleaner(test[0] % dirname)
             self.assertEqual(test[2], cleaner.auto_hide())
@@ -283,7 +284,7 @@ class WinappTestCase(unittest.TestCase, common.AssertFile):
                     "\nDetect1=HKCU\\Software\\does_not_exist1\nDetect2=HKCU\\Software\\does_not_exist1"):
                 new_ini = test[0] + detect
                 t = [new_ini, ] + [x for x in test[1:]]
-                print 'negative test', t
+                print('negative test', t)
                 # execute the test
                 (dirname, f1, f2, fbak) = self.setup_fake()
                 cleaner = self.ini2cleaner(t[0] % dirname, False)
@@ -402,11 +403,3 @@ class WinappTestCase(unittest.TestCase, common.AssertFile):
                  ('A - B (C)', 'a_b_c'))
         for test in tests:
             self.assertEqual(section2option(test[0]), test[1])
-
-
-def suite():
-    return unittest.makeSuite(WinappTestCase)
-
-
-if __name__ == '__main__':
-    unittest.main()
